from constructs import Construct
from aws_cdk import (
    App, 
    Stack,
    aws_sns as sns,
    aws_kms as kms
)


class MyStack(Stack):

    def __init__(self, scope: Construct, id: str, **kwargs) -> None:
        super().__init__(scope, id, **kwargs)

        topic = sns.Topic(self, "Topic",
                          topic_name="my-topic",
                          )

app = App()
MyStack(app, "MyStack")
app.synth()



import os
import warnings
from pathlib import Path
from typing import List
from parameterized import parameterized_class

from tests.graph_utils.utils import set_db_connector_by_graph_framework, PARAMETERIZED_GRAPH_FRAMEWORKS
from checkov.common.graph.graph_builder import CustomAttributes
from checkov.common.models.enums import CheckResult
from checkov.common.output.record import Record
from checkov.common.output.report import Report
from checkov.common.bridgecrew.check_type import CheckType
from checkov.common.runners.graph_manager import ObjectGraphManager
from checkov.github_actions.graph_builder.local_graph import GitHubActionsLocalGraph
from tests.common.graph.checks.test_yaml_policies_base import TestYamlPoliciesBase


@parameterized_class(PARAMETERIZED_GRAPH_FRAMEWORKS)
class TestYamlPolicies(TestYamlPoliciesBase):
    def __init__(self, args):
        db_connector = set_db_connector_by_graph_framework(self.graph_framework)

        graph_manager = ObjectGraphManager(db_connector=db_connector, source="GitHubActions")
        super().__init__(
            graph_manager=graph_manager,
            real_graph_checks_path=str(
                Path(__file__).parent.parent.parent.parent.parent / "checkov/github_actions/checks/graph_checks"
            ),
            test_checks_path="",
            check_type=CheckType.GITHUB_ACTIONS,
            test_file_path=__file__,
            args=args,
        )

    def setUp(self) -> None:
        warnings.filterwarnings("ignore", category=ResourceWarning)
        warnings.filterwarnings("ignore", category=DeprecationWarning)

    def test_ReadOnlyTopLevelPermissions(self):
        self.go("ReadOnlyTopLevelPermissions", local_graph_class=GitHubActionsLocalGraph)

    def test_registry_load(self):
        registry = self.get_checks_registry()
        self.assertGreater(len(registry.checks), 0)

    def assert_evaluated_keys(self, expected_evaluated_keys: List[str], results: List[Record]):
        evaluated_keys_results = results[0].check_result["evaluated_keys"]
        self.assertCountEqual(expected_evaluated_keys, evaluated_keys_results)

    def create_report_from_graph_checks_results(self, checks_results, check):
        report = Report(CheckType.GITHUB_ACTIONS)
        first_results_key = list(checks_results.keys())[0]
        for check_result in checks_results[first_results_key]:
            entity = check_result["entity"]
            record = Record(
                check_id=check["id"],
                check_name=check["name"],
                check_result=check_result,
                code_block=[(0, "")],
                file_path=entity.get(CustomAttributes.FILE_PATH),
                file_line_range=[entity.get("__startline__"), entity.get("__endline__")],
                resource=f"{entity.get(CustomAttributes.RESOURCE_TYPE)}.{entity.get(CustomAttributes.BLOCK_NAME)}",
                entity_tags=entity.get("tags", {}),
                evaluations=None,
                check_class="",
                file_abs_path=entity.get(CustomAttributes.FILE_PATH),
            )
            if check_result["result"] == CheckResult.PASSED:
                report.passed_checks.append(record)
            if check_result["result"] == CheckResult.FAILED:
                report.failed_checks.append(record)
        return report

    def assert_entities(self, expected_entities: List[str], results: List[Record], assertion: bool):
        self.assertEqual(len(expected_entities), len(results),
                         f"mismatch in number of results in {'passed' if assertion else 'failed'}, "
                         f"expected: {len(expected_entities)}, got: {len(results)}")
        for expected_entity in expected_entities:
            found = False
            for check_result in results:
                entity_id = f"{os.path.basename(check_result.file_path)}.{check_result.resource}"
                if entity_id == expected_entity:
                    found = True
                    break
            self.assertTrue(found, f"expected to find entity {expected_entity}, {'passed' if assertion else 'failed'}")



from checkov.cloudformation.checks.resource.base_resource_check import BaseResourceCheck
from checkov.common.models.enums import CheckResult, CheckCategories
from checkov.common.util.type_forcers import force_list


class IAMAdminPolicyDocument(BaseResourceCheck):
    def __init__(self):
        name = "Ensure no IAM policies that allow full \"*-*\" administrative privileges are not created"
        id = "CKV_AWS_62"
        supported_resources = ['AWS::IAM::Policy', 'AWS::IAM::Group', 'AWS::IAM::Role', 'AWS::IAM::User']
        categories = [CheckCategories.IAM]
        super().__init__(name=name, id=id, categories=categories, supported_resources=supported_resources)

    def scan_resource_conf(self, conf):
        my_properties = conf.get("Properties")
        type = conf['Type']

        # catch for inline policies
        if isinstance(my_properties, dict) and type != 'AWS::IAM::Policy':
            self.evaluated_keys = ['Properties']
            if 'Policies' in my_properties.keys():
                self.evaluated_keys = ['Properties/Policies']
                policies = my_properties['Policies']
                if len(policies) > 0:
                    for idx, policy in enumerate(policies):
                        if not isinstance(policy, dict):
                            return CheckResult.UNKNOWN
                        if policy.get('PolicyDocument'):
                            self.evaluated_keys = [f'Properties/Policies/[{idx}]/PolicyDocument']
                            result = check_policy(policy['PolicyDocument'])
                            if result == CheckResult.FAILED:
                                return result
                    return CheckResult.PASSED
                # not empty and had non failing policies
                return CheckResult.UNKNOWN
        # this is just for Policy resources
        if isinstance(my_properties, dict) and 'PolicyDocument' in my_properties.keys():
            self.evaluated_keys = ['Properties/PolicyDocument']
            return check_policy(my_properties['PolicyDocument'])
        return CheckResult.UNKNOWN


check = IAMAdminPolicyDocument()


def check_policy(policy_block):
    if policy_block and isinstance(policy_block, dict) and 'Statement' in policy_block.keys():
        for statement in force_list(policy_block['Statement']):
            if 'Action' in statement:
                effect = statement.get('Effect', 'Allow')
                action = force_list(statement.get('Action', ['']))
                resource = force_list(statement.get('Resource', ['']))
                if effect == 'Allow' and '*' in action and '*' in resource:
                    return CheckResult.FAILED
        return CheckResult.PASSED
    else:
        return CheckResult.PASSED



import unittest

import hcl2

from checkov.terraform.checks.resource.azure.SQLServerEmailAlertsToAdminsEnabled import check
from checkov.common.models.enums import CheckResult


class TestSQLServerEmailAlertsToAdminsEnabled(unittest.TestCase):

    def test_failure(self):
        hcl_res = hcl2.loads("""
            resource "azurerm_mssql_server_security_alert_policy" "example" {
              resource_group_name        = azurerm_resource_group.example.name
              server_name                = azurerm_sql_server.example.name
              state                      = "Enabled"
              storage_endpoint           = azurerm_storage_account.example.primary_blob_endpoint
              storage_account_access_key = azurerm_storage_account.example.primary_access_key
              disabled_alerts = [
                "Sql_Injection",
                "Data_Exfiltration"
              ]
              retention_days = 20
            }
                """)
        resource_conf = hcl_res['resource'][0]['azurerm_mssql_server_security_alert_policy']['example']
        scan_result = check.scan_resource_conf(conf=resource_conf)
        self.assertEqual(CheckResult.FAILED, scan_result)

    def test_success(self):
        hcl_res = hcl2.loads("""
            resource "azurerm_mssql_server_security_alert_policy" "example" {
              resource_group_name        = azurerm_resource_group.example.name
              server_name                = azurerm_sql_server.example.name
              state                      = "Enabled"
              storage_endpoint           = azurerm_storage_account.example.primary_blob_endpoint
              storage_account_access_key = azurerm_storage_account.example.primary_access_key
              disabled_alerts = []
              email_addresses = ["example@gmail.com"]
              email_account_admins = true
              retention_days = 20
            }
                """)
        resource_conf = hcl_res['resource'][0]['azurerm_mssql_server_security_alert_policy']['example']
        scan_result = check.scan_resource_conf(conf=resource_conf)
        self.assertEqual(CheckResult.PASSED, scan_result)


if __name__ == '__main__':
    unittest.main()



import unittest

from checkov.terraform import TFDefinitionKey
from checkov.terraform.tf_parser import TFParser
from checkov.terraform.context_parsers.registry import parser_registry
import os


class TestLocalsContextParser(unittest.TestCase):

    def setup_dir(self, rel_path):
        test_root_dir = os.path.dirname(os.path.realpath(__file__)) + rel_path
        parsing_errors = {}
        definitions_context = {}
        tf_definitions = TFParser().parse_directory(directory=test_root_dir,
                                 out_parsing_errors=parsing_errors)
        for definition in tf_definitions.items():
            definitions_context = parser_registry.enrich_definitions_context(definition)
        return definitions_context

    def test_assignments_exists(self):
        definitions_context = self.setup_dir('/../evaluation/resources/default_evaluation/')
        file_path = os.path.dirname(os.path.realpath(__file__)) + '/../evaluation/resources/default_evaluation/main.tf'
        key = TFDefinitionKey(file_path=file_path, tf_source_modules=None)
        assignments = definitions_context[key]['locals']['assignments']
        self.assertIsNotNone(assignments)

        expected_assignments = {'dummy_with_dash': '${format("-%s",var.dummy_1)}', 'dummy_with_comma': '${format(":%s",var.dummy_1)}', 'bucket_name': '${var.bucket_name}'}

        for k, v in assignments.items():
            self.assertEqual(expected_assignments[k], v)

    def test_assignment_value(self):
        definitions_context = self.setup_dir('/../evaluation/resources/locals_evaluation/')
        file_path = os.path.dirname(os.path.realpath(__file__)) + '/../evaluation/resources/locals_evaluation/main.tf'
        key = TFDefinitionKey(file_path=file_path, tf_source_modules=None)
        assignments = definitions_context[key]['locals'].get('assignments')
        self.assertIsNotNone(assignments)
        self.assertEqual(1, len(assignments.items()))
        for k, v in assignments.items():
            self.assertEqual(k, 'common_tags')
            self.assertIsInstance(v, dict)


if __name__ == '__main__':
    unittest.main()



"""External-checks signature verification."""
from __future__ import annotations

from .enforce import verify_external_checks_dirs
from .errors import SignatureVerificationError
from .keys import VerificationKey, load_public_keys
from .trailer_format import TRAILER_PREFIX, has_double_trailer
from .verifier import VerificationResult, verify_bytes, verify_file


__all__ = [
    "SignatureVerificationError",
    "TRAILER_PREFIX",
    "VerificationKey",
    "VerificationResult",
    "has_double_trailer",
    "load_public_keys",
    "verify_bytes",
    "verify_external_checks_dirs",
    "verify_file",
]



import os
import unittest

import pytest

from checkov.common.models.enums import CheckResult
from checkov.runner_filter import RunnerFilter
from checkov.terraform.checks.resource.aws.CloudfrontDistributionLogging import check
from checkov.terraform.runner import Runner


class TestCloudfrontDistributionLogging(unittest.TestCase):

    def test_failure(self):
        resource_conf = {
            "comment": "Example",
        }
        scan_result = check.scan_resource_conf(conf=resource_conf)
        self.assertEqual(CheckResult.FAILED, scan_result)

    def test_success(self):
        resource_conf = {
            "comment": "Example",
            "logging_config": [
                {
                    "bucket": "some-arn"
                }
            ],
        }
        scan_result = check.scan_resource_conf(conf=resource_conf)
        self.assertEqual(CheckResult.PASSED, scan_result)

    @pytest.mark.skip("Need to handle null variables")
    def test_null_var_651(self):
        self.skipTest("Need to handle null variables")
        current_dir = os.path.dirname(os.path.realpath(__file__))
        valid_dir_path = os.path.join(current_dir,
                                      "../../../parser/resources/parser_scenarios/null_variables_651")
        valid_dir_path = os.path.normpath(valid_dir_path)
        runner = Runner()
        checks_allowlist = ['CKV_AWS_86']
        report = runner.run(root_folder=valid_dir_path, external_checks_dir=None,
                            runner_filter=RunnerFilter(framework='terraform', checks=checks_allowlist))
        self.assertEqual(len(report.failed_checks), 1)
        for record in report.failed_checks:
            self.assertIn(record.check_id, checks_allowlist)


if __name__ == '__main__':
    unittest.main()



import json
import os
import platform
import sys
import unittest

current_dir = os.path.dirname(os.path.realpath(__file__))


class TestCheckovJsonReport(unittest.TestCase):
    def test_terragoat_report_dir_api_key(self):
        report_path = os.path.join(current_dir, '..', 'checkov_report_azuredir_api_key_terragoat.txt')
        self.validate_report(os.path.abspath(report_path))

    def test_terragoat_report_dir_no_upload_api_key(self):
        report_path = os.path.join(current_dir, '..', 'checkov_report_azuredir_api_key_terragoat_no_upload.txt')
        self.validate_report(os.path.abspath(report_path), False)

    def test_terragoat_report_file_api_key(self):
        report_path = os.path.join(current_dir, '..', 'checkov_report_s3_singlefile_api_key_terragoat.txt')
        self.validate_report(os.path.abspath(report_path))

    def validate_report(self, report_path, url_should_exist=True):
        if sys.version_info[1] == 8 and platform.system() == 'Linux':
            platform_url_found = False
            with open(report_path) as f:
                if 'More details: https://app0.prismacloud.io/projects?' in f.read():
                    platform_url_found = True
            self.assertEqual(platform_url_found, url_should_exist, "when using api key and not --skip-results-upload, platform code review url should exist")

    def test_bitbucket_pipelines_report_api_key(self):
        report_path = os.path.join(current_dir, '..', 'checkov_report_bitbucket_pipelines_cve.json')
        # the below condition exist because checkov_report_bitbucket_pipelines_cve.json is
        # generated only on Linux with Python 3.8 - see prepare_data.sh script
        if sys.version_info[1] == 8 and platform.system() == 'Linux':
            with open(report_path, encoding='utf-8') as f:
                report = json.load(f)
                self.assertGreaterEqual(len(report), 1,
                                        "expecting to have one report at least - bitbucket_pipelines ")
                bitbucket_pipelines_actions_report_exists = False
                if report["check_type"] == "bitbucket_pipelines":
                    bitbucket_pipelines_actions_report_exists = True
                    self.assertGreaterEqual(report['summary']['failed'], 1)

                self.assertTrue(bitbucket_pipelines_actions_report_exists)


if __name__ == '__main__':
    unittest.main()



import unittest
from pathlib import Path

from checkov.runner_filter import RunnerFilter
from checkov.terraform.checks.resource.azure.FrontDoorWAFACLCVE202144228 import check
from checkov.terraform.runner import Runner


class TestFrontDoorWAFACLCVE202144228(unittest.TestCase):
    def test(self):
        # given
        test_files_dir = Path(__file__).parent / "example_FrontDoorWAFACLCVE202144228"

        # when
        report = Runner().run(root_folder=str(test_files_dir), runner_filter=RunnerFilter(checks=[check.id]))

        # then
        summary = report.get_summary()

        passing_resources = {
            "azurerm_frontdoor_firewall_policy.dsr_1_1_default",
            "azurerm_frontdoor_firewall_policy.dsr_1_0_default",
            "azurerm_frontdoor_firewall_policy.dsr_1_1_enabled_block",
            "azurerm_frontdoor_firewall_policy.dsr_1_1_enabled_redirect",
        }
        failing_resources = {
            "azurerm_frontdoor_firewall_policy.default",
            "azurerm_frontdoor_firewall_policy.dsr_1_1_disabled",
            "azurerm_frontdoor_firewall_policy.dsr_1_1_disabled_default",
            "azurerm_frontdoor_firewall_policy.dsr_1_1_enabled_allow",
        }

        passed_check_resources = {c.resource for c in report.passed_checks}
        failed_check_resources = {c.resource for c in report.failed_checks}

        self.assertEqual(summary["passed"], 4)
        self.assertEqual(summary["failed"], 4)
        self.assertEqual(summary["skipped"], 0)
        self.assertEqual(summary["parsing_errors"], 0)

        self.assertEqual(passing_resources, passed_check_resources)
        self.assertEqual(failing_resources, failed_check_resources)

        # check especially for the evaluated keys
        actual_evaluated_keys = next(
            c.check_result["evaluated_keys"]
            for c in report.failed_checks
            if c.resource == "azurerm_frontdoor_firewall_policy.dsr_1_1_enabled_allow"
        )
        expected_evaluated_keys = [
            "managed_rule/[0]/type",
            "managed_rule/[0]/override/[0]/rule_group_name",
            "managed_rule/[0]/override/[0]/rule/[0]/rule_id",
            "managed_rule/[0]/override/[0]/rule/[0]/enabled",
            "managed_rule/[0]/override/[0]/rule/[0]/action",
        ]
        self.assertCountEqual(expected_evaluated_keys, actual_evaluated_keys)


if __name__ == "__main__":
    unittest.main()



from aws_cdk import core
from aws_cdk import aws_elasticloadbalancingv2 as elbv2

class MyALBWithAccessLogs(core.Stack):
    def __init__(self, scope: core.Construct, id: str, **kwargs) -> None:
        super().__init__(scope, id, **kwargs)

        # Define Elastic Load Balancer V2 with access logs enabled
        alb = elbv2.CfnLoadBalancer(
            self, 'MyALB',
            load_balancer_attributes=[
                elbv2.CfnLoadBalancer.LoadBalancerAttributeProperty(
                    key="access_logs.s3.enabled",
                    value="true"
                )
            ],
            # Other properties for your Application Load Balancer
        )

app = core.App()
MyALBWithAccessLogs(app, "MyALBWithAccessLogs")
app.synth()



"""``sys.path`` hygiene for the verified external-checks load path.

The verified load path is fastidious about cleanup elsewhere — it
uninstalls the meta-path finder, restores ``sys.dont_write_bytecode``,
and removes failed modules from ``sys.modules``. ``sys.path`` was the
one exception: every walked subdirectory was ``insert(1, root)``-ed
and never removed.

This is the regression test that the verified path leaves
``sys.path`` byte-identical to its pre-call snapshot, just like the
other module-level state.
"""
from __future__ import annotations

import sys
from pathlib import Path
from typing import Callable

import pytest
from ecdsa import SigningKey

from checkov.common.external_checks.verification.sources_registry import (
    reset_for_tests,
    verify_and_register,
)


@pytest.fixture(autouse=True)
def _reset_registry():
    reset_for_tests()
    try:
        yield
    finally:
        reset_for_tests()


@pytest.fixture(autouse=True)
def _restore_sys_path():
    """Belt-and-braces: never let one test's leak bleed into the next."""
    snapshot = list(sys.path)
    try:
        yield
    finally:
        sys.path[:] = snapshot


def _build_nested_signed_dir(
    root: Path, priv: SigningKey, make_trailer: Callable[[bytes, SigningKey], bytes],
) -> Path:
    """Two-level signed tree: ``root/__init__.py``, ``root/sub/__init__.py``,
    ``root/sub/aws_check.py`` — every ``.py`` carries a valid trailer.

    The nested level is what reproduces the leak: the walker hits both
    ``root`` and ``root/sub`` and inserts both into ``sys.path``.
    """
    root.mkdir()
    (root / "__init__.py").write_bytes(make_trailer(b"", priv))
    sub = root / "sub"
    sub.mkdir()
    (sub / "__init__.py").write_bytes(make_trailer(b"", priv))
    (sub / "aws_check.py").write_bytes(
        make_trailer(b"# signed nested check\nCHECK_ID = 'CKV_T_SP'\n", priv)
    )
    return root


def test_verified_load_does_not_leak_sys_path(
    tmp_path: Path,
    priv_a: SigningKey,
    key_a_pub_pem: bytes,
    make_trailer: Callable[[bytes, SigningKey], bytes],
    stub_registry,
):
    """After ``load_external_checks()`` returns, ``sys.path`` is unchanged.

    The verified path already restores ``sys.dont_write_bytecode`` and
    uninstalls the meta-path finder; ``sys.path`` is the missing piece.
    Walked directories must not persist on ``sys.path`` for the rest of
    the process lifetime.
    """
    checks_root = _build_nested_signed_dir(
        tmp_path / "verified_checks", priv_a, make_trailer,
    )

    key_path = tmp_path / "key.pem"
    key_path.write_bytes(key_a_pub_pem)

    # Register the directory through the same chokepoint the real code uses.
    verify_and_register([str(checks_root)], [str(key_path)])

    snapshot_before = list(sys.path)
    stub_registry.load_external_checks(str(checks_root))
    snapshot_after = list(sys.path)

    leaked = [p for p in snapshot_after if p not in snapshot_before]
    assert leaked == [], (
        f"verified load path leaked {len(leaked)} entries onto sys.path: "
        f"{leaked!r}. The verified path already restores "
        "sys.dont_write_bytecode and uninstalls the meta-path finder in a "
        "finally block; sys.path must be restored to its pre-call snapshot "
        "in the same finally block so external-check loading is hermetic."
    )



import unittest
from pathlib import Path

from checkov.dockerfile.checks.WorkdirIsAbsolute import check
from checkov.dockerfile.runner import Runner
from checkov.runner_filter import RunnerFilter


class TestWorkdirIsAbsolute(unittest.TestCase):
    def test(self):
        # given
        test_files_dir = Path(__file__).parent / "example_WorkdirIsAbsolute"

        # when
        report = Runner().run(root_folder=str(test_files_dir), runner_filter=RunnerFilter(checks=[check.id]))

        # then
        summary = report.get_summary()

        passing_resources = {"/success/Dockerfile."}
        failing_resources = [
            "/failure/Dockerfile.WORKDIR",  # filenames appear once for each check violation in the file
            "/failure/Dockerfile.WORKDIR",
            "/failure/Dockerfile.WORKDIR",
            "/failure/Dockerfile.simple.WORKDIR",
        ]

        passed_check_resources = {c.resource for c in report.passed_checks}
        failed_check_resources = [c.resource for c in report.failed_checks]

        self.assertEqual(summary["passed"], len(passing_resources))
        self.assertEqual(summary["failed"], len(failing_resources))
        self.assertEqual(summary["skipped"], 0)
        self.assertEqual(summary["parsing_errors"], 0)

        self.assertEqual(passing_resources, passed_check_resources)
        self.assertCountEqual(failing_resources, failed_check_resources)


if __name__ == "__main__":
    unittest.main()



from aws_cdk import core
from aws_cdk import aws_s3 as s3

class S3BucketWithPublicAccessStack(core.Stack):

    def __init__(self, scope: core.Construct, id: str, **kwargs) -> None:
        super().__init__(scope, id, **kwargs)
        s3.Bucket(
            self,
            "aaa",
            versioned=False,  # You can enable versioning if needed
            removal_policy=core.RemovalPolicy.DESTROY,  # Change this according to your retention policy
            block_public_acls=True,
            block_public_policy=True,
            ignore_public_acls=True,
            restrict_public_buckets=True
        )

app = core.App()
S3BucketWithPublicAccessStack(app, "S3BucketWithPublicAccessStack")
app.synth()

class PublicS3BucketStack(core.Stack):

    def __init__(self, scope: core.Construct, id: str, **kwargs) -> None:
        super().__init__(scope, id, **kwargs)

        # Create a CloudFormation S3 bucket resource
        public_bucket = s3.CfnBucket(
            self,
            "PublicBucket",
            versioning_configuration={
                "status": "Suspended"  # You can enable versioning if needed
            },
            public_access_block_configuration={
                "blockPublicAcls": True,
                "blockPublicPolicy": True,
                "ignorePublicAcls": True,
                "restrictPublicBuckets": True
            }
        )

app = core.App()
PublicS3BucketStack(app, "PublicS3BucketStack")
app.synth()




from __future__ import annotations

import os
import unittest
import warnings
from pathlib import Path

from checkov.common.checks_infra.checks_parser import GraphCheckParser
from checkov.common.models.enums import CheckResult
from checkov.common.checks_infra.registry import Registry
from .test_yaml_policies import load_yaml_data, get_policy_results


class TestCustomYamlPolicies(unittest.TestCase):
    def setUp(self) -> None:
        os.environ['UNIQUE_TAG'] = ''
        warnings.filterwarnings("ignore", category=ResourceWarning)
        warnings.filterwarnings("ignore", category=DeprecationWarning)

    def test_CustomPolicy1(self):
        self.go("CustomPolicy1")

    def test_CustomPolicy2(self):
        # tests resource_types value to be a string
        self.go("CustomPolicy2")

    def test_CustomAwsEMRSecurityConfiguration(self):
        self.go('CustomAwsEMRSecurityConfiguration')

    def go(self, dir_name: str, check_name: str | None = None) -> None:
        dir_path = os.path.join(os.path.dirname(os.path.realpath(__file__)), f"resources/{dir_name}")
        check_name = dir_name if check_name is None else check_name
        assert os.path.exists(dir_path)
        policy_dir_path = str(Path(__file__).parent / "custom_policies")
        assert os.path.exists(policy_dir_path)
        found = False
        for root, d_names, f_names in os.walk(policy_dir_path):
            for f_name in f_names:
                if f_name == f"{check_name}.yaml":
                    found = True
                    policy = load_yaml_data(f_name, root)
                    assert policy is not None
                    expected = load_yaml_data("expected.yaml", dir_path)
                    assert expected is not None
                    registry = Registry(policy_dir_path, GraphCheckParser())
                    report = get_policy_results(dir_path, [policy['metadata']['id']], [registry])
                    expected = load_yaml_data("expected.yaml", dir_path)

                    expected_to_fail = expected.get('fail', [])
                    expected_to_pass = expected.get('pass', [])
                    expected_to_skip = expected.get('skip', [])

                    self.assert_entities(expected_to_pass, report.passed_checks, True)
                    self.assert_entities(expected_to_fail, report.failed_checks, False)
                    self.assert_entities(expected_to_skip, report.skipped_checks, True)

        assert found

    def assert_entities(self, expected_entities: list[str], results: list[CheckResult], assertion: bool) -> None:
        self.assertEqual(len(expected_entities), len(results),
                         f"mismatch in number of results in {'passed' if assertion else 'failed'}, "
                         f"expected: {len(expected_entities)}, got: {len(results)}")
        for expected_entity in expected_entities:
            found = False
            for check_result in results:
                entity_id = check_result.resource
                if entity_id == expected_entity:
                    found = True
                    break
            self.assertTrue(found, f"expected to find entity {expected_entity}, {'passed' if assertion else 'failed'}")



import logging
from unittest import mock

from checkov.common.resource_code_logger_filter import add_resource_code_filter_to_logger


TEST_CODE_TEMPLATES_TO_REPLACE = "THIS-SHOULD-BE-REPLACED!"


def test_code_logger_filter_do_not_log_if_not_allowed(caplog) -> None:
    with mock.patch("checkov.common.resource_code_logger_filter.ResourceCodeFilter.CODE_TEMPLATES",
                    [TEST_CODE_TEMPLATES_TO_REPLACE]):
        logger = logging.getLogger("code logging not allowed")
        add_resource_code_filter_to_logger(logger, allow_code_logging=False)
        logger.warning(TEST_CODE_TEMPLATES_TO_REPLACE)
        assert TEST_CODE_TEMPLATES_TO_REPLACE not in caplog.text


def test_code_logger_filter_logs_if_allowed(caplog) -> None:
    with mock.patch("checkov.common.resource_code_logger_filter.ResourceCodeFilter.CODE_TEMPLATES",
                    [TEST_CODE_TEMPLATES_TO_REPLACE]):
        logger = logging.getLogger("code logging allowed")
        add_resource_code_filter_to_logger(logger)
        logger.warning(TEST_CODE_TEMPLATES_TO_REPLACE)
        assert TEST_CODE_TEMPLATES_TO_REPLACE in caplog.text


def test_code_logger_filter_logs_based_on_arg_not_allowed(caplog) -> None:
    with mock.patch("checkov.common.resource_code_logger_filter.ResourceCodeFilter.CODE_TEMPLATES",
                    [TEST_CODE_TEMPLATES_TO_REPLACE]):
        logger = logging.getLogger("code logging not allowed")
        add_resource_code_filter_to_logger(logger, allow_code_logging=False)
        logger.warning(TEST_CODE_TEMPLATES_TO_REPLACE, extra={"mask": True})
        assert TEST_CODE_TEMPLATES_TO_REPLACE not in caplog.text



import unittest

from pytest_mock import MockerFixture

from checkov.docs_generator import get_compare_key, get_check_link


class TestOutputSorting(unittest.TestCase):

    def test_id_sorting_for_ckv_pattern(self):
        # keep all other things the same so sorting is based on the id
        check_ids = [
            ['CKV_AWS_1', '', '', '', ''],
            ['CKV_K8S_11', '', '', '', ''],
            ['CKV_K8S_15', '', '', '', ''],
            ['CKV_K8S_9', '', '', '', ''],
            ['CKV_K8S_2', '', '', '', ''],
            ['CKV2_K8S_2', '', '', '', ''],
            ['CKV_AZURE_11', '', '', '', ''],
            ['CKV_AZURE_32', '', '', '', ''],
            ['CKV_GIT_1', '', '', '', ''],
            ['CKV_AZURE_10', '', '', '', ''],
            ['CKV2_AWS_1', '', '', '', ''],
            ['CKV_AZURE_22', '', '', '', ''],
            ['CKV_K8S_20', '', '', '', ''],
            ['CKV_GCP_1', '', '', '', ''],
            ['CKV_K8S_1', '', '', '', ''],
            ['CKV_GCP_10', '', '', '', ''],
            ['CKV_AZURE_12', '', '', '', ''],
            ['CKV_K8S_10', '', '', '', ''],
            ['CKV_AWS_20', '', '', '', ''],
        ]
        sorted_check_ids = sorted(check_ids, key=get_compare_key)
        self.assertEqual(sorted_check_ids, [
            ['CKV_AWS_1', '', '', '', ''],
            ['CKV_AWS_20', '', '', '', ''],
            ['CKV2_AWS_1', '', '', '', ''],
            ['CKV_AZURE_10', '', '', '', ''],
            ['CKV_AZURE_11', '', '', '', ''],
            ['CKV_AZURE_12', '', '', '', ''],
            ['CKV_AZURE_22', '', '', '', ''],
            ['CKV_AZURE_32', '', '', '', ''],
            ['CKV_GCP_1', '', '', '', ''],
            ['CKV_GCP_10', '', '', '', ''],
            ['CKV_GIT_1', '', '', '', ''],
            ['CKV_K8S_1', '', '', '', ''],
            ['CKV_K8S_2', '', '', '', ''],
            ['CKV_K8S_9', '', '', '', ''],
            ['CKV_K8S_10', '', '', '', ''],
            ['CKV_K8S_11', '', '', '', ''],
            ['CKV_K8S_15', '', '', '', ''],
            ['CKV_K8S_20', '', '', '', ''],
            ['CKV2_K8S_2', '', '', '', ''],
        ])

    def test_sorting_by_resource_id(self):
        checks_list = [
            ['CKV_AWS_1', '', 'aws_ebs_volume', '', ''],
            ['CKV_AWS_1', '', 'AWS::EBS::Volume', '', ''],
            ['CKV_AWS_1', '', 'AWS::S3::Bucket', '', ''],
            ['CKV_AWS_1', '', 'aws_s3_bucket', '', '']
        ]

        sorted_list = sorted(checks_list, key=get_compare_key)

        self.assertEqual(sorted_list[0], ['CKV_AWS_1', '', 'AWS::EBS::Volume', '', ''])
        self.assertEqual(sorted_list[1], ['CKV_AWS_1', '', 'AWS::S3::Bucket', '', ''])
        self.assertEqual(sorted_list[2], ['CKV_AWS_1', '', 'aws_ebs_volume', '', ''])
        self.assertEqual(sorted_list[3], ['CKV_AWS_1', '', 'aws_s3_bucket', '', ''])


def test_get_check_link():
    # given
    abs_path = "/path/to/checkov/checkov/terraform/checks/resource/aws/LambdaXrayEnabled.py"

    # when
    link = get_check_link(absolute_path=abs_path)

    # then
    assert link == "https://github.com/bridgecrewio/checkov/blob/main/checkov/terraform/checks/resource/aws/LambdaXrayEnabled.py"


def test_get_check_link_for_markdown(mocker: MockerFixture):
    # given
    abs_path = "/path/to/checkov/checkov/terraform/checks/resource/aws/LambdaXrayEnabled.py"

    mocker.patch("checkov.docs_generator.CREATE_MARKDOWN_HYPERLINKS", return_value=True)

    # when
    link = get_check_link(absolute_path=abs_path)

    # then
    assert link == "[LambdaXrayEnabled.py](https://github.com/bridgecrewio/checkov/blob/main/checkov/terraform/checks/resource/aws/LambdaXrayEnabled.py)"


if __name__ == '__main__':
    unittest.main()



import unittest

from checkov.serverless.registry import sls_registry


class TestScannerRegistry(unittest.TestCase):

    def test_num_of_scanners(self):
        scanners_counter = 0
        for key in list(sls_registry.checks.keys()):
            scanners_counter += len(sls_registry.checks[key])

        self.assertGreater(scanners_counter, 1)

    def test_non_colliding_check_ids(self):
        check_id_check_class_map = {}
        for (resource_type, checks) in sls_registry.checks.items():
            for check in checks:
                check_id_check_class_map.setdefault(check.id, []).append(check)

        for check_id, check_classes in check_id_check_class_map.items():
            self.assertEqual(len(set(check_classes)), 1)


if __name__ == '__main__':
    unittest.main()



from pathlib import Path

from pytest_mock import MockerFixture

from checkov.common.bridgecrew.bc_source import get_source_type
from checkov.common.bridgecrew.check_type import CheckType
from checkov.dockerfile.runner import Runner
from checkov.runner_filter import RunnerFilter
from tests.common.image_referencer.test_utils import mock_get_license_statuses_async, mock_get_image_cached_result_async

RESOURCES_PATH = Path(__file__).parent / "resources"


def test_simple_dockerfile(mocker: MockerFixture):
    from checkov.common.bridgecrew.platform_integration import bc_integration

    # given
    file_name = "Dockerfile.simple"
    image_name = "php:7.1-apache"
    code_lines = "1-1"
    test_file = RESOURCES_PATH / file_name
    runner_filter = RunnerFilter(run_image_referencer=True)
    bc_integration.bc_source = get_source_type("disabled")

    mocker.patch(
        "checkov.common.images.image_referencer.image_scanner.get_scan_results_from_cache_async",
        side_effect=mock_get_image_cached_result_async,
    )
    mocker.patch(
        "checkov.common.images.image_referencer.get_license_statuses_async",
        side_effect=mock_get_license_statuses_async,
    )

    # when
    reports = Runner().run(root_folder="", files=[str(test_file)], runner_filter=runner_filter)

    # then
    assert len(reports) == 2

    tf_report = next(report for report in reports if report.check_type == CheckType.DOCKERFILE)
    sca_image_report = next(report for report in reports if report.check_type == CheckType.SCA_IMAGE)

    assert len(tf_report.resources) == 1
    assert len(tf_report.passed_checks) == 21
    assert len(tf_report.failed_checks) == 2
    assert len(tf_report.skipped_checks) == 0
    assert len(tf_report.parsing_errors) == 0

    assert len(sca_image_report.resources) == 3
    assert sca_image_report.resources == {
        f"{file_name} ({image_name} lines:{code_lines} (sha256:2460522297)).musl",
        f"{file_name} ({image_name} lines:{code_lines} (sha256:2460522297)).openssl",
        f"{file_name} ({image_name} lines:{code_lines} (sha256:2460522297)).go",
    }
    assert sca_image_report.image_cached_results[0]["dockerImageName"] == "php:7.1-apache"
    assert "Dockerfile.simple.FROM" in sca_image_report.image_cached_results[0]["relatedResourceId"]
    assert sca_image_report.image_cached_results[0]["packages"] == [
        {"type": "os", "name": "tzdata", "version": "2021a-1+deb11u5", "licenses": []}
    ]

    assert len(sca_image_report.passed_checks) == 1
    assert len(sca_image_report.failed_checks) == 4
    assert len(sca_image_report.image_cached_results) == 1
    assert len(sca_image_report.skipped_checks) == 0
    assert len(sca_image_report.parsing_errors) == 0


def test_multi_stage_dockerfile(mocker: MockerFixture):
    from checkov.common.bridgecrew.platform_integration import bc_integration

    # given
    file_name = "Dockerfile.multi_stage"
    image_name_1 = "maven:3.8-openjdk-17-slim"
    image_name_2 = "amazonlinux:2"
    code_lines_1 = "2-2"
    code_lines_2 = "10-10"
    test_file = RESOURCES_PATH / file_name
    runner_filter = RunnerFilter(run_image_referencer=True)
    bc_integration.bc_source = get_source_type("disabled")

    mocker.patch(
        "checkov.common.images.image_referencer.image_scanner.get_scan_results_from_cache_async",
        side_effect=mock_get_image_cached_result_async,
    )

    # when
    reports = Runner().run(root_folder="", files=[str(test_file)], runner_filter=runner_filter)

    # then
    assert len(reports) == 2

    tf_report = next(report for report in reports if report.check_type == CheckType.DOCKERFILE)
    sca_image_report = next(report for report in reports if report.check_type == CheckType.SCA_IMAGE)

    assert len(tf_report.resources) == 1
    assert len(tf_report.passed_checks) == 38
    assert len(tf_report.failed_checks) == 2
    assert len(tf_report.skipped_checks) == 0
    assert len(tf_report.parsing_errors) == 0

    assert len(sca_image_report.resources) == 1
    assert sca_image_report.resources == {
        f"{file_name} ({image_name_2} lines:{code_lines_2} (sha256:2460522297)).go",
    }
    assert len(sca_image_report.passed_checks) == 0
    assert len(sca_image_report.failed_checks) == 3
    assert len(sca_image_report.skipped_checks) == 0
    assert len(sca_image_report.parsing_errors) == 0


def test_multi_platform_dockerfile(mocker: MockerFixture):
    from checkov.common.bridgecrew.platform_integration import bc_integration

    # given
    file_name = "Dockerfile.multi_platform"
    image_name_1 = "golang:alpine"
    image_name_2 = "alpine"
    code_lines_1 = "1-1"
    code_lines_2 = "8-8"
    test_file = RESOURCES_PATH / file_name
    runner_filter = RunnerFilter(run_image_referencer=True)
    bc_integration.bc_source = get_source_type("disabled")

    mocker.patch(
        "checkov.common.images.image_referencer.image_scanner.get_scan_results_from_cache_async",
        side_effect=mock_get_image_cached_result_async,
    )

    # when
    reports = Runner().run(root_folder="", files=[str(test_file)], runner_filter=runner_filter)

    # then
    assert len(reports) == 2

    tf_report = next(report for report in reports if report.check_type == CheckType.DOCKERFILE)
    sca_image_report = next(report for report in reports if report.check_type == CheckType.SCA_IMAGE)

    assert len(tf_report.resources) == 1
    assert len(tf_report.passed_checks) == 30
    assert len(tf_report.failed_checks) == 3
    assert len(tf_report.skipped_checks) == 0
    assert len(tf_report.parsing_errors) == 0

    assert len(sca_image_report.resources) == 1
    assert sca_image_report.resources == {
        f"{file_name} ({image_name_2} lines:{code_lines_2} (sha256:2460522297)).go",
    }
    assert len(sca_image_report.passed_checks) == 0
    assert len(sca_image_report.failed_checks) == 3
    assert len(sca_image_report.skipped_checks) == 0
    assert len(sca_image_report.parsing_errors) == 0



import unittest

import hcl2

from checkov.terraform.checks.resource.azure.RedisCachePublicNetworkAccessEnabled import check
from checkov.common.models.enums import CheckResult


class TestRedisCachePublicNetworkAccessEnabled(unittest.TestCase):

    def test_failure(self):
        hcl_res = hcl2.loads("""
                    resource "azurerm_redis_cache" "example" {
                      name                = "example-cache"
                      location            = azurerm_resource_group.example.location
                      resource_group_name = azurerm_resource_group.example.name
                      capacity            = 2
                      family              = "C"
                      sku_name            = "Standard"
                      enable_non_ssl_port = false
                      minimum_tls_version = "1.2"
                      public_network_access_enabled  = true
                      redis_configuration {
                      }
                    }
                """)
        resource_conf = hcl_res['resource'][0]['azurerm_redis_cache']['example']
        scan_result = check.scan_resource_conf(conf=resource_conf)
        self.assertEqual(CheckResult.FAILED, scan_result)

    def test_failure_no_param(self):
        hcl_res = hcl2.loads("""
                    resource "azurerm_redis_cache" "example" {
                      name                = "example-cache"
                      location            = azurerm_resource_group.example.location
                      resource_group_name = azurerm_resource_group.example.name
                      capacity            = 2
                      family              = "C"
                      sku_name            = "Standard"
                      enable_non_ssl_port = false
                      minimum_tls_version = "1.2"

                      redis_configuration {
                      }
                    }
                """)
        resource_conf = hcl_res['resource'][0]['azurerm_redis_cache']['example']
        scan_result = check.scan_resource_conf(conf=resource_conf)
        self.assertEqual(CheckResult.FAILED, scan_result)

    def test_success(self):
        hcl_res = hcl2.loads("""
                    resource "azurerm_redis_cache" "example" {
                      name                = "example-cache"
                      location            = azurerm_resource_group.example.location
                      resource_group_name = azurerm_resource_group.example.name
                      capacity            = 2
                      family              = "C"
                      sku_name            = "Standard"
                      enable_non_ssl_port = false
                      minimum_tls_version = "1.2"
                      public_network_access_enabled  = false

                      redis_configuration {
                      }
                    }
                """)
        resource_conf = hcl_res['resource'][0]['azurerm_redis_cache']['example']
        scan_result = check.scan_resource_conf(conf=resource_conf)
        self.assertEqual(CheckResult.PASSED, scan_result)


if __name__ == '__main__':
    unittest.main()



import unittest

import hcl2

from checkov.terraform.checks.resource.azure.CutsomRoleDefinitionSubscriptionOwner import check
from checkov.common.models.enums import CheckResult


class TestCustomRoleDefinitionSubscriptionOwner(unittest.TestCase):

    def test_failure_1(self):
        hcl_res = hcl2.loads("""
            resource "azurerm_role_definition" "example" {
              name        = "my-custom-role"
              scope       = data.azurerm_subscription.primary.id
              description = "This is a custom role created via Terraform"
            
              permissions {
                actions     = ["*"]
                not_actions = []
              }
            
              assignable_scopes = [
                data.azurerm_subscription.primary.id
              ]
            }
                """)
        resource_conf = hcl_res['resource'][0]['azurerm_role_definition']['example']
        scan_result = check.scan_resource_conf(conf=resource_conf)
        self.assertEqual(CheckResult.FAILED, scan_result)

    def test_failure_2(self):
        hcl_res = hcl2.loads("""
            resource "azurerm_role_definition" "example" {
              name        = "my-custom-role"
              scope       = data.azurerm_subscription.primary.id
              description = "This is a custom role created via Terraform"

              permissions {
                actions     = ["*"]
                not_actions = []
              }

              assignable_scopes = [
                "/"
              ]
            }
                """)
        resource_conf = hcl_res['resource'][0]['azurerm_role_definition']['example']
        scan_result = check.scan_resource_conf(conf=resource_conf)
        self.assertEqual(CheckResult.FAILED, scan_result)

    def test_success(self):
        hcl_res = hcl2.loads("""
            resource "azurerm_role_definition" "example" {
              name        = "my-custom-role"
              scope       = data.azurerm_subscription.primary.id
              description = "This is a custom role created via Terraform"
            
              permissions {
                actions     = [
                "Microsoft.Authorization/*/read",
                  "Microsoft.Insights/alertRules/*",
                  "Microsoft.Resources/deployments/write",
                  "Microsoft.Resources/subscriptions/operationresults/read",
                  "Microsoft.Resources/subscriptions/read",
                  "Microsoft.Resources/subscriptions/resourceGroups/read",
                  "Microsoft.Support/*"
                  ]
                not_actions = []
              }
            
              assignable_scopes = [
                data.azurerm_subscription.primary.id
              ]
            }
                """)
        resource_conf = hcl_res['resource'][0]['azurerm_role_definition']['example']
        scan_result = check.scan_resource_conf(conf=resource_conf)
        self.assertEqual(CheckResult.PASSED, scan_result)

    def test_no_assignable_scopes(self):
        hcl_res = hcl2.loads("""
            resource "azurerm_role_definition" "example" {
              name        = "my-custom-role"
              scope       = data.azurerm_subscription.primary.id
              description = "This is a custom role created via Terraform"

              permissions {
                actions     = ["*"]
                not_actions = []
              }
            }
                """)
        resource_conf = hcl_res['resource'][0]['azurerm_role_definition']['example']
        scan_result = check.scan_resource_conf(conf=resource_conf)
        self.assertEqual(CheckResult.FAILED, scan_result)


if __name__ == '__main__':
    unittest.main()



from __future__ import annotations

import asyncio
import sys


def mock_get_empty_license_statuses_async(packages, image_name: str):
    result = {'image_name': image_name, 'licenses': []}

    if sys.version_info < (3, 8):
        future = asyncio.Future()
        future.set_result(result)
        return future

    return result


def mock_get_license_statuses_async(packages, image_name: str) -> dict[str, str | list[dict[str, str]]]:
    result = {
        "image_name": image_name,
        "licenses": [
            {
                "package_name": "openssl",
                "package_version": "1.1.1q-r0",
                "policy": "BC_LIC_1",
                "license": "OpenSSL",
                "status": "OPEN",
            },
            {
                "package_name": "musl",
                "package_version": "1.2.3-r0",
                "policy": "BC_LIC_1",
                "license": "MIT",
                "status": "COMPLIANT",
            },
        ]
    }

    if sys.version_info < (3, 8):
        future = asyncio.Future()
        future.set_result(result)
        return future

    return result


def mock_get_image_cached_result_async(image_id: str):
    result = {
        "results": [
            {
                "id": "sha256:2460522297a148c1bcb477b126451ed44cca05c916694367313be1a91c69f793",
                "name": "redis:latest",
                "distro": "Debian GNU/Linux 11 (bullseye)",
                "distroRelease": "bullseye",
                "digest": "sha256:091a7b5de688f283b30a4942280b64cf822bbdab0abfb2d2ce6db989f2d3c3f4",
                "collections": [
                    "All"
                ],
                "packages": [
                    {
                        "type": "os",
                        "name": "tzdata",
                        "version": "2021a-1+deb11u5"
                    }
                ],
                "compliances": [
                    {
                        "id": 41,
                        "title": "(CIS_Docker_v1.2.0 - 4.1) Image should be created with a non-root user",
                        "severity": "high",
                        "description": "It is a good practice to run the container as a non-root user, if possible. Though user\nnamespace mapping is now available, if a user is already defined in the container image, the\ncontainer is run as that user by default and specific user namespace remapping is not\nrequired"
                    }
                ],
                "complianceDistribution": {
                    "critical": 0,
                    "high": 1,
                    "medium": 0,
                    "low": 0,
                    "total": 1
                },
                "complianceScanPassed": "true",
                "vulnerabilities": [
                    {
                        "id": "CVE-2021-38297",
                        "status": "fixed in 1.17.2, 1.16.9",
                        "cvss": 9.8,
                        "vector": "CVSS:3.1/AV:N/AC:L/PR:N/UI:N/S:U/C:H/I:H/A:H",
                        "description": "Go before 1.16.9 and 1.17.x before 1.17.2 has a Buffer Overflow via large arguments in a function invocation from a WASM module, when GOARCH\u003dwasm GOOS\u003djs is used.",
                        "severity": "critical",
                        "packageName": "go",
                        "packageVersion": "1.16.7",
                        "link": "https://nvd.nist.gov/vuln/detail/CVE-2021-38297",
                        "riskFactors": [
                            "Attack complexity: low",
                            "Attack vector: network",
                            "Critical severity",
                            "Has fix",
                            "Recent vulnerability"
                        ],
                        "impactedVersions": [
                            "\u003c1.16.9"
                        ],
                        "publishedDate": "2021-10-18T06:15:00Z",
                        "discoveredDate": "2022-09-18T14:26:20Z",
                        "fixDate": "2021-10-18T06:15:00Z"
                    },
                    {
                        "id": "CVE-2022-23806",
                        "status": "fixed in 1.17.7, 1.16.14",
                        "cvss": 9.1,
                        "vector": "CVSS:3.1/AV:N/AC:L/PR:N/UI:N/S:U/C:N/I:H/A:H",
                        "description": "Curve.IsOnCurve in crypto/elliptic in Go before 1.16.14 and 1.17.x before 1.17.7 can incorrectly return true in situations with a big.Int value that is not a valid field element.",
                        "severity": "critical",
                        "packageName": "go",
                        "packageVersion": "1.16.7",
                        "link": "https://nvd.nist.gov/vuln/detail/CVE-2022-23806",
                        "riskFactors": [
                            "Recent vulnerability",
                            "Attack complexity: low",
                            "Attack vector: network",
                            "Critical severity",
                            "Has fix"
                        ],
                        "impactedVersions": [
                            "\u003c1.16.14"
                        ],
                        "publishedDate": "2022-02-11T01:15:00Z",
                        "discoveredDate": "2022-09-18T14:26:20Z",
                        "fixDate": "2022-02-11T01:15:00Z"
                    },
                    {
                        "id": "CVE-2022-30580",
                        "status": "fixed in 1.18.3, 1.17.11",
                        "cvss": 7.8,
                        "vector": "CVSS:3.1/AV:L/AC:L/PR:L/UI:N/S:U/C:H/I:H/A:H",
                        "description": "Code injection in Cmd.Start in os/exec before Go 1.17.11 and Go 1.18.3 allows execution of any binaries in the working directory named either \\\"..com\\\" or \\\"..exe\\\" by calling Cmd.Run, Cmd.Start, Cmd.Output, or Cmd.CombinedOutput when Cmd.Path is unset.",
                        "severity": "high",
                        "packageName": "go",
                        "packageVersion": "1.16.7",
                        "link": "https://nvd.nist.gov/vuln/detail/CVE-2022-30580",
                        "riskFactors": [
                            "Attack complexity: low",
                            "Has fix",
                            "High severity",
                            "Recent vulnerability"
                        ],
                        "impactedVersions": [
                            "\u003c1.17.11"
                        ],
                        "publishedDate": "2022-08-10T20:15:00Z",
                        "discoveredDate": "2022-09-18T14:26:20Z",
                        "fixDate": "2022-08-10T20:15:00Z"
                    }
                ],
                "vulnerabilityDistribution": {
                    "critical": 2,
                    "high": 19,
                    "medium": 5,
                    "low": 12,
                    "total": 38
                },
                "vulnerabilityScanPassed": "true"
            }
        ]
    }

    if sys.version_info < (3, 8):
        future = asyncio.Future()
        future.set_result(result)
        return future

    return result



import json
import os
import unittest

current_dir = os.path.dirname(os.path.realpath(__file__))


class TestCheckovSarifReport(unittest.TestCase):

    def test_terragoat_report(self):
        report_path = os.path.join(os.path.dirname(current_dir), 'results.sarif')
        self.validate_report(os.path.abspath(report_path))

    def validate_report(self, report_path):
        with open(report_path) as json_file:
            data = json.load(json_file)
            if isinstance(data, list):
                for framework_report in data:
                    self.validate_report_not_empty(framework_report)
            else:
                self.validate_report_not_empty(data)

    def validate_report_not_empty(self, report):
        self.assertEqual(report["runs"][0]['tool']['driver']['name'], "Checkov")
        self.assertGreater(len(report["runs"][0]['results']), 1,
                           "expecting more than 1 failed checks")


if __name__ == '__main__':
    unittest.main()



import unittest

import hcl2

from checkov.terraform.checks.resource.azure.VMEncryptionAtHostEnabled import check
from checkov.common.models.enums import CheckResult


class TestVMEncryptionAtHostEnabled(unittest.TestCase):

    def test_failure1(self):
        hcl_res = hcl2.loads("""
             resource "azurerm_windows_virtual_machine_scale_set" "example" {
                  name                = "example-vmss"
                  resource_group_name = azurerm_resource_group.example.name
                  location            = azurerm_resource_group.example.location
                  sku                 = "Standard_F2"
                  instances           = 1
                  admin_password      = "P@55w0rd1234!"  # checkov:skip=CKV_SECRET_80 test secret
                  admin_username      = "adminuser"
                
                  source_image_reference {
                    publisher = "MicrosoftWindowsServer"
                    offer     = "WindowsServer"
                    sku       = "2016-Datacenter-Server-Core"
                    version   = "latest"
                  }
                
                  os_disk {
                    storage_account_type = "Standard_LRS"
                    caching              = "ReadWrite"
                  }
                
                  network_interface {
                    name    = "example"
                    primary = true
                
                    ip_configuration {
                      name      = "internal"
                      primary   = true
                      subnet_id = azurerm_subnet.internal.id
                    }
                  }
                }   """)
        resource_conf = hcl_res['resource'][0]['azurerm_windows_virtual_machine_scale_set']['example']
        scan_result = check.scan_resource_conf(conf=resource_conf)
        self.assertEqual(CheckResult.FAILED, scan_result)

    def test_failure2(self):
        hcl_res = hcl2.loads("""
             resource "azurerm_windows_virtual_machine_scale_set" "example" {
                  name                = "example-vmss"
                  resource_group_name = azurerm_resource_group.example.name
                  location            = azurerm_resource_group.example.location
                  sku                 = "Standard_F2"
                  instances           = 1
                  admin_password      = "P@55w0rd1234!"
                  admin_username      = "adminuser"
                  encryption_at_host_enabled = false

                  source_image_reference {
                    publisher = "MicrosoftWindowsServer"
                    offer     = "WindowsServer"
                    sku       = "2016-Datacenter-Server-Core"
                    version   = "latest"
                  }

                  os_disk {
                    storage_account_type = "Standard_LRS"
                    caching              = "ReadWrite"
                  }

                  network_interface {
                    name    = "example"
                    primary = true

                    ip_configuration {
                      name      = "internal"
                      primary   = true
                      subnet_id = azurerm_subnet.internal.id
                    }
                  }
                }   """)
        resource_conf = hcl_res['resource'][0]['azurerm_windows_virtual_machine_scale_set']['example']
        scan_result = check.scan_resource_conf(conf=resource_conf)
        self.assertEqual(CheckResult.FAILED, scan_result)

    def test_failure3(self):
        hcl_res = hcl2.loads("""
             resource "azurerm_linux_virtual_machine_scale_set" "example" {
                  name                = "example-vmss"
                  resource_group_name = azurerm_resource_group.example.name
                  location            = azurerm_resource_group.example.location
                  sku                 = "Standard_F2"
                  instances           = 1
                  admin_password      = "P@55w0rd1234!"
                  admin_username      = "adminuser"

                  source_image_reference {
                    publisher = "MicrosoftWindowsServer"
                    offer     = "WindowsServer"
                    sku       = "2016-Datacenter-Server-Core"
                    version   = "latest"
                  }

                  os_disk {
                    storage_account_type = "Standard_LRS"
                    caching              = "ReadWrite"
                  }

                  network_interface {
                    name    = "example"
                    primary = true

                    ip_configuration {
                      name      = "internal"
                      primary   = true
                      subnet_id = azurerm_subnet.internal.id
                    }
                  }
                }   """)
        resource_conf = hcl_res['resource'][0]['azurerm_linux_virtual_machine_scale_set']['example']
        scan_result = check.scan_resource_conf(conf=resource_conf)
        self.assertEqual(CheckResult.FAILED, scan_result)

    def test_failure4(self):
        hcl_res = hcl2.loads("""
             resource "azurerm_linux_virtual_machine_scale_set" "example" {
                  name                = "example-vmss"
                  resource_group_name = azurerm_resource_group.example.name
                  location            = azurerm_resource_group.example.location
                  sku                 = "Standard_F2"
                  instances           = 1
                  admin_password      = "P@55w0rd1234!"
                  admin_username      = "adminuser"
                  encryption_at_host_enabled = false

                  source_image_reference {
                    publisher = "MicrosoftWindowsServer"
                    offer     = "WindowsServer"
                    sku       = "2016-Datacenter-Server-Core"
                    version   = "latest"
                  }

                  os_disk {
                    storage_account_type = "Standard_LRS"
                    caching              = "ReadWrite"
                  }

                  network_interface {
                    name    = "example"
                    primary = true

                    ip_configuration {
                      name      = "internal"
                      primary   = true
                      subnet_id = azurerm_subnet.internal.id
                    }
                  }
                }   """)
        resource_conf = hcl_res['resource'][0]['azurerm_linux_virtual_machine_scale_set']['example']
        scan_result = check.scan_resource_conf(conf=resource_conf)
        self.assertEqual(CheckResult.FAILED, scan_result)

    def test_success1(self):
        hcl_res = hcl2.loads("""
             resource "azurerm_windows_virtual_machine_scale_set" "example" {
                  name                = "example-vmss"
                  resource_group_name = azurerm_resource_group.example.name
                  location            = azurerm_resource_group.example.location
                  sku                 = "Standard_F2"
                  instances           = 1
                  admin_password      = "P@55w0rd1234!"
                  admin_username      = "adminuser"
                  encryption_at_host_enabled = true

                  source_image_reference {
                    publisher = "MicrosoftWindowsServer"
                    offer     = "WindowsServer"
                    sku       = "2016-Datacenter-Server-Core"
                    version   = "latest"
                  }

                  os_disk {
                    storage_account_type = "Standard_LRS"
                    caching              = "ReadWrite"
                  }

                  network_interface {
                    name    = "example"
                    primary = true

                    ip_configuration {
                      name      = "internal"
                      primary   = true
                      subnet_id = azurerm_subnet.internal.id
                    }
                  }
                }   """)
        resource_conf = hcl_res['resource'][0]['azurerm_windows_virtual_machine_scale_set']['example']
        scan_result = check.scan_resource_conf(conf=resource_conf)
        self.assertEqual(CheckResult.PASSED, scan_result)

    def test_success2(self):
        hcl_res = hcl2.loads("""
             resource "azurerm_linux_virtual_machine_scale_set" "example" {
                  name                = "example-vmss"
                  resource_group_name = azurerm_resource_group.example.name
                  location            = azurerm_resource_group.example.location
                  sku                 = "Standard_F2"
                  instances           = 1
                  admin_password      = "P@55w0rd1234!"
                  admin_username      = "adminuser"
                  encryption_at_host_enabled = true

                  source_image_reference {
                    publisher = "MicrosoftWindowsServer"
                    offer     = "WindowsServer"
                    sku       = "2016-Datacenter-Server-Core"
                    version   = "latest"
                  }

                  os_disk {
                    storage_account_type = "Standard_LRS"
                    caching              = "ReadWrite"
                  }

                  network_interface {
                    name    = "example"
                    primary = true

                    ip_configuration {
                      name      = "internal"
                      primary   = true
                      subnet_id = azurerm_subnet.internal.id
                    }
                  }
                }   """)
        resource_conf = hcl_res['resource'][0]['azurerm_linux_virtual_machine_scale_set']['example']
        scan_result = check.scan_resource_conf(conf=resource_conf)
        self.assertEqual(CheckResult.PASSED, scan_result)


if __name__ == '__main__':
    unittest.main()



import unittest

import hcl2

from checkov.terraform.checks.resource.gcp.GoogleComputeDiskEncryption import check
from checkov.common.models.enums import CheckResult


class TestGoogleComputeDiskEncryption(unittest.TestCase):

    def test_failure(self):
        hcl_res = hcl2.loads("""
            resource "google_compute_disk" "default" {
              name  = "test-disk"
              type  = "pd-ssd"
              zone  = "us-central1-a"
              image = "debian-8-jessie-v20170523"
              physical_block_size_bytes = 4096
            }
                """)
        resource_conf = hcl_res['resource'][0]['google_compute_disk']['default']
        scan_result = check.scan_resource_conf(conf=resource_conf)
        self.assertEqual(CheckResult.FAILED, scan_result)

    def test_success(self):
        hcl_res = hcl2.loads("""
            resource "google_compute_disk" "default" {
              name  = "test-disk"
              type  = "pd-ssd"
              zone  = "us-central1-a"
              image = "debian-8-jessie-v20170523"
              physical_block_size_bytes = 4096
              disk_encryption_key {
                raw_key = "acXTX3rxrKAFTF0tYVLvydU1riRZTvUNC4g5I11NY-c="  # checkov:skip=CKV_SECRET_6 test secret
                }
            }
                """)
        resource_conf = hcl_res['resource'][0]['google_compute_disk']['default']
        scan_result = check.scan_resource_conf(conf=resource_conf)
        self.assertEqual(CheckResult.PASSED, scan_result)


if __name__ == '__main__':
    unittest.main()



import logging

from checkov.common.goget.base_getter import BaseGetter


class LocalGetter(BaseGetter):
    def __init__(self, url):
        self.logger = logging.getLogger(__name__)
        super().__init__(url)

    def do_get(self):

        return self.temp_dir



from __future__ import annotations

import importlib.util
import logging
import os
import sys
from importlib.abc import MetaPathFinder
from importlib.machinery import ModuleSpec, SourceFileLoader
from types import ModuleType
from typing import Iterable, Mapping, Sequence


logger = logging.getLogger(__name__)


class _VerifiedSourceLoader(SourceFileLoader):
    """Serves pre-verified in-memory bytes through the stdlib import machinery."""

    def __init__(self, fullname: str, file_path: str, source_bytes: bytes) -> None:
        super().__init__(fullname, file_path)
        self._source_bytes = source_bytes

    def get_data(self, path: str) -> bytes:
        return self._source_bytes

    def path_stats(self, path: str) -> "Mapping[str, int]":
        # Fixed mtime/size so the stdlib does not stat() the file or consult
        # an on-disk .pyc cache (which would not be re-verified).
        return {"mtime": 0, "size": len(self._source_bytes)}

    def get_source(self, fullname: str) -> str:
        return self._source_bytes.decode("utf-8", errors="replace")

    def set_data(  # type: ignore[override]
        self, path: str, data: bytes, *, _mode: int = 0o666,
    ) -> None:
        # Opt out of stdlib .pyc cache writes: those bytes would have no
        # trailer of their own and could be picked up by a later process.
        return None

    def exec_module(self, module: ModuleType) -> None:
        # Drop the half-initialised module on failure so a subsequent import
        # of the same name gets ModuleNotFoundError instead of a partial.
        try:
            super().exec_module(module)
        except BaseException:
            sys.modules.pop(module.__name__, None)
            raise


def load_verified_sources_into_module(
    module_name: str,
    file_path: str,
    source_bytes: bytes,
) -> ModuleType:
    """Load ``source_bytes`` as ``module_name`` via the stdlib import machinery.

    Pre-registers in ``sys.modules`` BEFORE ``exec_module`` so self-imports
    resolve. Cleans up on failure.
    """
    loader = _VerifiedSourceLoader(module_name, file_path, source_bytes)
    spec = importlib.util.spec_from_loader(module_name, loader, origin=file_path)
    if spec is None:
        raise ImportError(f"cannot create module spec for {module_name} at {file_path}")
    module = importlib.util.module_from_spec(spec)
    # spec_from_loader doesn't propagate __file__ onto the module; external
    # checks rely on it for diagnostics, so set it explicitly.
    module.__file__ = file_path
    sys.modules[module_name] = module
    try:
        loader.exec_module(module)
    except BaseException:
        sys.modules.pop(module_name, None)
        raise
    return module


class VerifiedSourcesFinder(MetaPathFinder):
    """v1 scope: top-level (un-dotted) module names only."""

    def __init__(
        self,
        verified_sources: Mapping[str, bytes],
        verified_dirs: Sequence[str],
    ) -> None:
        self._verified_sources = dict(verified_sources)
        self._verified_dirs = tuple(verified_dirs)

    def find_spec(
        self,
        fullname: str,
        path: "Sequence[str] | None" = None,
        target: "object | None" = None,
    ) -> "ModuleSpec | None":
        if "." in fullname:
            return None
        candidate_name = fullname + ".py"
        for verified_dir in self._verified_dirs:
            candidate_path = os.path.join(verified_dir, candidate_name)
            if candidate_path in self._verified_sources:
                source_bytes = self._verified_sources[candidate_path]
                loader = _VerifiedSourceLoader(fullname, candidate_path, source_bytes)
                return importlib.util.spec_from_loader(fullname, loader, origin=candidate_path)
        return None


def install_finder(
    verified_sources: Mapping[str, bytes],
    verified_dirs: Iterable[str],
) -> VerifiedSourcesFinder:
    finder = VerifiedSourcesFinder(verified_sources, list(verified_dirs))
    sys.meta_path.insert(0, finder)
    return finder


def uninstall_finder(finder: VerifiedSourcesFinder) -> None:
    """Idempotent — second call is a silent no-op.

    Uses identity-based removal (``x is not finder``) instead of
    ``sys.meta_path.remove(finder)``. ``list.remove`` is ``__eq__``-based
    and walks every element until it finds a match; if any earlier
    MetaPathFinder on ``sys.meta_path`` has a user-defined ``__eq__``
    that hangs (e.g. waiting on a lock held by another thread), the
    whole removal hangs with it. Filtering by ``is`` avoids any
    ``__eq__`` invocation. We rebuild the list in place so any other
    references to ``sys.meta_path`` keep seeing the same list object.
    """
    sys.meta_path[:] = [x for x in sys.meta_path if x is not finder]


__all__ = [
    "VerifiedSourcesFinder",
    "install_finder",
    "load_verified_sources_into_module",
    "uninstall_finder",
]



from aws_cdk import core
from aws_cdk import aws_elasticloadbalancingv2 as elbv2

class MyListenerStack(core.Stack):
    def __init__(self, scope: core.Construct, id: str, **kwargs) -> None:
        super().__init__(scope, id, **kwargs)

        # Define HTTPS Redirect Listener
        listener = elbv2.CfnListener(
            self, 'MyHTTPSRedirectListener',
            load_balancer_arn='your-load-balancer-arn',  # Replace with your ALB ARN
            protocol='HTTP',
            port=80,
            default_actions=[{
                'type': 'abc',
                'redirectConfig': {
                    'protocol': 'HTTP',
                }
            }]
            # Other properties for your Redirect Listener
        )

app = core.App()
MyListenerStack(app, "MyListenerStack")
app.synth()



import unittest

import hcl2

from checkov.terraform.checks.resource.azure.KeyVaultEnablesFirewallRulesSettings import check
from checkov.common.models.enums import CheckResult


class TestKeyVaultEnablesFirewallRulesSettings(unittest.TestCase):

    def test_failure_missing(self):
        hcl_res = hcl2.loads("""
                resource "azurerm_key_vault" "example" {
                  name                        = "examplekeyvault"
                  location                    = azurerm_resource_group.example.location
                  resource_group_name         = azurerm_resource_group.example.name
                  enabled_for_disk_encryption = true
                  tenant_id                   = data.azurerm_client_config.current.tenant_id
                  soft_delete_retention_days  = 7
                  purge_protection_enabled    = false
                
                  sku_name = "standard"
                
                  access_policy {
                    tenant_id = data.azurerm_client_config.current.tenant_id
                    object_id = data.azurerm_client_config.current.object_id
                
                    key_permissions = [
                      "Get",
                    ]
                    
                    secret_permissions = [
                      "Get",
                    ]
                
                    storage_permissions = [
                      "Get",
                    ]
                  }
                }
                """)
        resource_conf = hcl_res['resource'][0]['azurerm_key_vault']['example']
        scan_result = check.scan_resource_conf(conf=resource_conf)
        self.assertEqual(CheckResult.FAILED, scan_result)

    def test_success(self):
        hcl_res = hcl2.loads("""
                resource "azurerm_key_vault" "example" {
                  name                        = "examplekeyvault"
                  location                    = azurerm_resource_group.example.location
                  resource_group_name         = azurerm_resource_group.example.name
                  enabled_for_disk_encryption = true
                  tenant_id                   = data.azurerm_client_config.current.tenant_id
                  soft_delete_retention_days  = 7
                  purge_protection_enabled    = false
                
                  sku_name = "standard"
                
                  access_policy {
                    tenant_id = data.azurerm_client_config.current.tenant_id
                    object_id = data.azurerm_client_config.current.object_id
                
                    key_permissions = [
                      "Get",
                    ]
                    
                    secret_permissions = [
                      "Get",
                    ]
                
                    storage_permissions = [
                      "Get",
                    ]
                  }
                  network_acls {
                    default_action = "Deny"
                  }
                }
                """)
        resource_conf = hcl_res['resource'][0]['azurerm_key_vault']['example']
        scan_result = check.scan_resource_conf(conf=resource_conf)
        self.assertEqual(CheckResult.PASSED, scan_result)

    def test_fail_allow(self):
        hcl_res = hcl2.loads("""
                resource "azurerm_key_vault" "example" {
                  name                        = "examplekeyvault"
                  location                    = azurerm_resource_group.example.location
                  resource_group_name         = azurerm_resource_group.example.name
                  enabled_for_disk_encryption = true
                  tenant_id                   = data.azurerm_client_config.current.tenant_id
                  soft_delete_retention_days  = 7
                  purge_protection_enabled    = false
                
                  sku_name = "standard"
                
                  access_policy {
                    tenant_id = data.azurerm_client_config.current.tenant_id
                    object_id = data.azurerm_client_config.current.object_id
                
                    key_permissions = [
                      "Get",
                    ]
                
                    secret_permissions = [
                      "Get",
                    ]
                
                    storage_permissions = [
                      "Get",
                    ]
                  }
                  network_acls {
                    default_action = "Allow"
                  }
                }
                """)
        resource_conf = hcl_res['resource'][0]['azurerm_key_vault']['example']
        scan_result = check.scan_resource_conf(conf=resource_conf)
        self.assertEqual(CheckResult.FAILED, scan_result)


if __name__ == '__main__':
    unittest.main()



import unittest

from checkov.common.models.enums import CheckResult
from checkov.terraform.checks.resource.azure.AKSNetworkPolicy import check


class TestAKSNetworkPolicy(unittest.TestCase):

    def test_failure(self):
        resource_conf = {'name': ['example-aks1'], 'location': ['${azurerm_resource_group.example.location}'],
                         'resource_group_name': ['${azurerm_resource_group.example.name}'], 'dns_prefix': ['exampleaks1'],
                         'default_node_pool': [{'name': ['default'], 'node_count': [1], 'vm_size': ['Standard_D2_v2']}],
                         'identity': [{'type': ['SystemAssigned']}], 'agent_pool_profile': [{}], 'service_principal': [{}],
                         'api_server_authorized_ip_ranges': [['192.168.0.0/16']], 'tags': [{'Environment': 'Production'}],
                         'addon_profile': [{'oms_agent': [{'enabled': [True], 'log_analytics_workspace_id': ['']}]}]}

        scan_result = check.scan_resource_conf(conf=resource_conf)
        self.assertEqual(CheckResult.FAILED, scan_result)

    def test_failure2(self):
        resource_conf = {'name': ['example-aks1'], 'location': ['${azurerm_resource_group.example.location}'],
                         'resource_group_name': ['${azurerm_resource_group.example.name}'], 'dns_prefix': ['exampleaks1'],
                         'default_node_pool': [{'name': ['default'], 'node_count': [1], 'vm_size': ['Standard_D2_v2']}],
                         'identity': [{'type': ['SystemAssigned']}], 'agent_pool_profile': [{}], 'service_principal': [{}],
                         'api_server_authorized_ip_ranges': [[]], 'role_based_access_control': [{'enabled': [True]}],
                         'network_profile': [{'network_plugin': ['azure']}], 'tags': [{'Environment': 'Production'}]}

        scan_result = check.scan_resource_conf(conf=resource_conf)
        self.assertEqual(CheckResult.FAILED, scan_result)

    def test_success(self):
        resource_conf = {'name': ['example-aks1'], 'location': ['${azurerm_resource_group.example.location}'],
                         'resource_group_name': ['${azurerm_resource_group.example.name}'], 'dns_prefix': ['exampleaks1'],
                         'default_node_pool': [{'name': ['default'], 'node_count': [1], 'vm_size': ['Standard_D2_v2']}],
                         'identity': [{'type': ['SystemAssigned']}], 'agent_pool_profile': [{}], 'service_principal': [{}],
                         'role_based_access_control': [{'enabled': [False]}],
                         'network_profile': [{'network_plugin': ['azure'], 'network_policy': ['network_policy']}],
                         'tags': [{'Environment': 'Production'}]}
        scan_result = check.scan_resource_conf(conf=resource_conf)
        self.assertEqual(CheckResult.PASSED, scan_result)


if __name__ == '__main__':
    unittest.main()



import unittest

from checkov.common.models.enums import CheckResult
from checkov.terraform.checks.resource.aws.AthenaDatabaseEncryption import check


class TestAthenaDatabaseEncryption(unittest.TestCase):

    def test_failure(self):
        resource_conf = {
            "name": "Example",
        }
        scan_result = check.scan_resource_conf(conf=resource_conf)
        self.assertEqual(CheckResult.FAILED, scan_result)

    def test_success(self):
        resource_conf = {
            "name": "Example",
            "encryption_configuration": [
                {
                    "encryption_option": "SSE_S3"
                }
            ],
        }

        scan_result = check.scan_resource_conf(conf=resource_conf)
        self.assertEqual(CheckResult.PASSED, scan_result)


if __name__ == '__main__':
    unittest.main()



import pytest
from checkov.github_actions.runner import Runner


@pytest.mark.parametrize(
    "start_line,end_line,expected_job_name",
    [
        (10, 15, "container-test-job"),
        (8, 20, "container-test-job"),
        (24, 30, "second_job"),
        (25, 27, "second_job"),
        (5, 40, "")
    ],
)
def test_resolve_job_name(start_line, end_line, expected_job_name, definition):
    job_name = Runner.resolve_sub_name(definition, start_line, end_line, tag='jobs')

    assert job_name == expected_job_name


@pytest.mark.parametrize(
    "key,expected_key, supported_entities, start_line, end_line",
    [
        ('jobs.container-test-job.CKV_GHA_3[7:23]', "jobs(container-test-job)",
         ('jobs', 'jobs.*.steps[]'), 7, 23),
        ('jobs.*.steps[].jobs.*.steps[].CKV_GHA_3[18:23]', "jobs(container-test-job).steps[1](Check for dockerenv file)",
         ('jobs', 'jobs.*.steps[]'), 18, 23),
        ('jobs.*.steps[].jobs.*.steps[].CKV_GHA_3[31:35]', "jobs(no_step_name_job).steps[1]",
        ('jobs', 'jobs.*.steps[]'), 31, 35),
    ],
)
def test_get_resource(key, supported_entities, expected_key, start_line, end_line, definition):
    runner = Runner()
    file_path = "mock_path"
    runner.definitions[file_path] = definition

    new_key = runner.get_resource(file_path, key, supported_entities, start_line, end_line)

    assert new_key == expected_key



from pathlib import Path

from checkov.common.graph.db_connectors.networkx.networkx_db_connector import NetworkxConnector
from checkov.common.graph.graph_builder import CustomAttributes
from checkov.common.graph.graph_builder.graph_components.block_types import BlockType
from checkov.common.runners.graph_manager import ObjectGraphManager
from checkov.ansible.graph_builder.local_graph import AnsibleLocalGraph
from checkov.ansible.runner import Runner
from checkov.common.util.consts import START_LINE, END_LINE

EXAMPLES_DIR = Path(__file__).parent / "examples"


def test_build_graph_from_definitions():
    # given
    test_file = str(EXAMPLES_DIR / "site.yml")
    graph_manager = ObjectGraphManager(db_connector=NetworkxConnector(), source="Ansible")
    template, _ = Runner()._parse_file(f=test_file)

    # when
    local_graph = graph_manager.build_graph_from_definitions(
        definitions={test_file: template}, graph_class=AnsibleLocalGraph
    )

    # then
    assert len(local_graph.vertices) == 2

    task_idx = local_graph.vertices_by_path_and_name[(test_file, "tasks.amazon.aws.ec2_instance.enabled")]
    task = local_graph.vertices[task_idx]

    assert task.block_type == BlockType.RESOURCE
    assert task.id == "tasks.amazon.aws.ec2_instance.enabled"
    assert task.source == "Ansible"
    assert task.attributes[CustomAttributes.RESOURCE_TYPE] == "tasks.amazon.aws.ec2_instance"
    assert task.attributes[START_LINE] == 11
    assert task.attributes[END_LINE] == 22
    assert task.config == {'name': 'enabled',
                           'amazon.aws.ec2_instance':
                               {'name': 'public-compute-instance',
                                'key_name': 'prod-ssh-key',
                                'vpc_subnet_id': 'subnet-5ca1ab1e',
                                'instance_type': 'c5.large',
                                'security_group': 'default',
                                'network': {'assign_public_ip': True, '__startline__': 19, '__endline__': 20},
                                'image_id': 'ami-123456',
                                'ebs_optimized': True,
                                '__startline__': 13,
                                '__endline__': 22},
                           '__startline__': 11,
                           '__endline__': 22}



import unittest

from checkov.terraform.checks.resource.gcp.GKEClientCertificateDisabled import check
from checkov.common.models.enums import CheckResult


class GKEClientCertificateDisabled(unittest.TestCase):

    def test_success(self):
        resource_conf = {'name': ['google_cluster'], 'master_auth': [{'client_certificate_config': [{'issue_client_certificate': [False]}]}]}

        scan_result = check.scan_resource_conf(conf=resource_conf)
        self.assertEqual(CheckResult.PASSED, scan_result)

    def test_failure(self):
        resource_conf = {'name': ['google_cluster'], 'master_auth': [{'client_certificate_config': [{'issue_client_certificate': [True]}]}]}
        scan_result = check.scan_resource_conf(conf=resource_conf)
        self.assertEqual(CheckResult.FAILED, scan_result)


if __name__ == '__main__':
    unittest.main()



import unittest

import hcl2

from checkov.terraform.checks.resource.azure.PostgreSQLServerLogConnectionsEnabled import check
from checkov.common.models.enums import CheckResult


class TestPostgreSQLServerLogConnectionsEnabled(unittest.TestCase):

    def test_failure(self):
        hcl_res = hcl2.loads("""
            resource "azurerm_postgresql_configuration" "example" {
              name                = "log_connections"
              resource_group_name = data.azurerm_resource_group.example.name
              server_name         = azurerm_postgresql_server.example.name
              value               = "off"
            }
                """)
        resource_conf = hcl_res['resource'][0]['azurerm_postgresql_configuration']['example']
        scan_result = check.scan_resource_conf(conf=resource_conf)
        self.assertEqual(CheckResult.FAILED, scan_result)

    def test_success(self):
        hcl_res = hcl2.loads("""
            resource "azurerm_postgresql_configuration" "example" {
              name                = "backslash_quote"
              resource_group_name = azurerm_resource_group.example.name
              server_name         = azurerm_postgresql_server.example.name
              value               = "on"
            }
                """)
        resource_conf = hcl_res['resource'][0]['azurerm_postgresql_configuration']['example']
        scan_result = check.scan_resource_conf(conf=resource_conf)
        self.assertEqual(CheckResult.PASSED, scan_result)


if __name__ == '__main__':
    unittest.main()



from checkov.common.models.enums import CheckResult, CheckCategories
from checkov.terraform.checks.resource.base_resource_check import BaseResourceCheck


class S3EnvironmentCheck(BaseResourceCheck):
    def __init__(self):
        name = "Ensure s3 has env tag of development/staging/production"
        id = "CUSTOM_AWS_1"
        supported_resources = ['aws_s3_bucket']
        categories = [CheckCategories.GENERAL_SECURITY]
        super().__init__(name=name, id=id, categories=categories, supported_resources=supported_resources)

    def scan_resource_conf(self, conf):
        if conf.get("tags") and isinstance(conf['tags'][0], dict):
            env = conf["tags"][0].get("Environment",{})
            if env in ["Development","Staging","Production"]:
                return CheckResult.PASSED
        return CheckResult.FAILED


scanner = S3EnvironmentCheck()



from checkov.common.sca.commons import normalize_twistcli_language


def test_normalize_twistcli_language_for_gem():
    assert normalize_twistcli_language("gem") == "ruby"


def test_normalize_twistcli_language_for_ruby():
    assert normalize_twistcli_language("ruby") == "ruby"


def test_normalize_twistcli_language_for_empty():
    assert normalize_twistcli_language("") == ""


def test_normalize_twistcli_language_for_invalid():
    assert normalize_twistcli_language("bbb") == "bbb"


def test_normalize_twistcli_language_for_python():
    assert normalize_twistcli_language("python") == "python"



from __future__ import annotations

import datetime
import linecache
import logging
import os
import re
import tempfile
import threading
from contextlib import contextmanager
from copy import deepcopy
from pathlib import Path
from typing import TYPE_CHECKING, cast, Optional, Iterable, Any, List, Dict, Tuple, Callable, Generator
from collections import defaultdict

import requests
from detect_secrets.filters.heuristic import is_potential_uuid
from detect_secrets.settings import (
    get_settings,
    configure_settings_from_baseline,
    get_plugins,
    get_filters,
)
from detect_secrets.core.plugins.util import get_mapping_from_secret_type_to_class

from checkov.common.util.decorators import time_it
from checkov.common.util.type_forcers import convert_str_to_bool

from checkov.common.bridgecrew.platform_integration import bc_integration
from checkov.common.output.secrets_record import SecretsRecord
from checkov.common.util.http_utils import request_wrapper, DEFAULT_TIMEOUT
from detect_secrets import SecretsCollection
from detect_secrets.core import scan
from detect_secrets.core.potential_secret import PotentialSecret

from checkov.common.bridgecrew.check_type import CheckType
from checkov.common.bridgecrew.integration_features.features.policy_metadata_integration import \
    integration as metadata_integration
from checkov.common.bridgecrew.severities import Severity
from checkov.common.comment.enum import COMMENT_REGEX
from checkov.common.models.consts import SUPPORTED_FILE_EXTENSIONS
from checkov.common.models.enums import CheckResult
from checkov.common.output.report import Report
from checkov.common.parallelizer.parallel_runner import parallel_runner
from checkov.common.runners.base_runner import BaseRunner, filter_ignored_paths
from checkov.common.typing import _CheckResult
from checkov.common.util.dockerfile import is_dockerfile
from checkov.common.util.secrets import omit_secret_value_from_line, GENERIC_OBFUSCATION_LENGTH
from checkov.runner_filter import RunnerFilter
from checkov.common.secrets.consts import ValidationStatus, VerifySecretsResult
from checkov.secrets.coordinator import EnrichedSecret, SecretsCoordinator
from checkov.secrets.plugins.load_detectors import get_runnable_plugins
from checkov.secrets.git_history_store import GitHistorySecretStore
from checkov.secrets.git_types import EnrichedPotentialSecret, PROHIBITED_FILES, Commit
from checkov.secrets.scan_git_history import GitHistoryScanner
from checkov.secrets.utils import filter_excluded_paths, EXCLUDED_PATHS
from checkov.secrets.context_parser import ContextParser
from checkov.secrets.log_prefix_stripper import create_stripped_content

if TYPE_CHECKING:
    from checkov.common.util.tqdm_utils import ProgressBar

SOURCE_CODE_EXTENSION = ['.py', '.js', '.properties', '.pem', '.php', '.xml', '.ts', '.env', '.java', '.rb',
                         'go', 'cs', '.txt']
SECRET_TYPE_TO_ID = {
    'Artifactory Credentials': 'CKV_SECRET_1',
    'AWS Access Key': 'CKV_SECRET_2',
    'Azure Storage Account access key': 'CKV_SECRET_3',
    'Basic Auth Credentials': 'CKV_SECRET_4',
    'Cloudant Credentials': 'CKV_SECRET_5',
    'Base64 High Entropy String': 'CKV_SECRET_6',
    'IBM Cloud IAM Key': 'CKV_SECRET_7',
    'IBM COS HMAC Credentials': 'CKV_SECRET_8',
    'JSON Web Token': 'CKV_SECRET_9',
    'Secret Keyword': 'CKV_SECRET_10',
    'Mailchimp Access Key': 'CKV_SECRET_11',
    'NPM tokens': 'CKV_SECRET_12',
    'Private Key': 'CKV_SECRET_13',
    'Slack Token': 'CKV_SECRET_14',
    'SoftLayer Credentials': 'CKV_SECRET_15',
    'Square OAuth Secret': 'CKV_SECRET_16',
    'Stripe Access Key': 'CKV_SECRET_17',
    'Twilio API Key': 'CKV_SECRET_18',
    'Hex High Entropy String': 'CKV_SECRET_19'
}

BASE64_HIGH_ENTROPY_CHECK_ID = 'CKV_SECRET_6'
RANDOM_HIGH_ENTROPY_CHECK_ID = 'CKV_SECRET_80'
ENTROPY_CHECK_IDS = {BASE64_HIGH_ENTROPY_CHECK_ID, 'CKV_SECRET_19', RANDOM_HIGH_ENTROPY_CHECK_ID}
GENERIC_PRIVATE_KEY_CHECK_IDS = {'CKV_SECRET_4', 'CKV_SECRET_9', 'CKV_SECRET_10', 'CKV_SECRET_13', 'CKV_SECRET_192'}
GENERIC_AWS_CHECK_ID = 'CKV_SECRET_2'
SPECIFIC_AWS_CHECK_IDS = {'CKV_SECRET_380', 'CKV_SECRET_381'}

CHECK_ID_TO_SECRET_TYPE = {v: k for k, v in SECRET_TYPE_TO_ID.items()}

MAX_FILE_SIZE = int(os.getenv('CHECKOV_MAX_FILE_SIZE', '5000000'))  # 5 MB is default limit

# ── thread-safe replacement for transient_settings ─────────────────
_detect_secrets_settings_lock = threading.Lock()


@contextmanager
def _thread_safe_transient_settings(config: Dict[str, Any]) -> Generator[Any, None, None]:
    """Thread-safe replacement for detect_secrets.settings.transient_settings.

    The original transient_settings calls cache_bust() which clears the
    get_settings() LRU cache, creating a race window where get_settings()
    returns a fresh empty Settings() object. Sub-threads can then cache
    a corrupted (empty) plugin mapping, resulting in 0 findings.

    This implementation never clears get_settings() from the LRU cache.
    Instead it modifies the Settings singleton in-place and only clears
    the dependent caches (get_plugins, get_filters,
    get_mapping_from_secret_type_to_class).

    The lock is held only for microseconds during setup and teardown —
    never during the actual scan — so there is no performance impact.

    Pre-warming: after reconfiguring, we call get_mapping_from_secret_type_to_class()
    and get_plugins() while still holding the lock. This populates both LRU caches
    with the correct plugin set before any inner worker thread can observe them.
    Without pre-warming, inner threads race to rebuild the caches concurrently after
    the lock is released — Python's @lru_cache is not thread-safe for concurrent
    cache misses, so two threads can simultaneously enter the function body and one
    can overwrite the other's result with a corrupted (empty) mapping.
    """
    with _detect_secrets_settings_lock:
        settings = get_settings()
        original_plugins = deepcopy(dict(settings.plugins))
        original_filters = deepcopy(dict(settings.filters))

        # Clear ONLY the dependent caches — NOT get_settings itself.
        # This is the key difference from the original transient_settings:
        # the singleton object is never evicted from the LRU cache.
        get_plugins.cache_clear()
        get_filters.cache_clear()
        get_mapping_from_secret_type_to_class.cache_clear()

        settings.clear()
        configure_settings_from_baseline(config)

        # Pre-warm both caches while the lock is still held so that inner
        # worker threads spawned by parallel_runner always get cache hits
        # during the scan phase and never need to rebuild the mapping.
        # Without this, concurrent cache misses in @lru_cache (not thread-safe)
        # can produce a corrupted empty mapping → TypeError → 0 findings.
        mapping = get_mapping_from_secret_type_to_class()
        plugins = get_plugins()
        logging.debug(
            f"_thread_safe_transient_settings (with pre-warming): settings_id={id(settings)} plugins_count={len(plugins)} mapping_count={len(mapping)} thread={__import__('threading').current_thread().name}")

    # Lock released — scan runs freely here with warm, stable caches
    try:
        yield settings
    finally:
        with _detect_secrets_settings_lock:
            get_plugins.cache_clear()
            get_filters.cache_clear()
            get_mapping_from_secret_type_to_class.cache_clear()
            settings.plugins = original_plugins
            settings.filters = original_filters


def should_filter_vault_secret(secret_value: str, check_id: str) -> bool:
    return 'vault:' in secret_value.lower() and check_id in ENTROPY_CHECK_IDS


def _get_secret_suppressions_ids() -> List[str]:
    secret_suppressions_ids: list[str] = []
    if bc_integration.customer_run_config_response:
        suppressions = bc_integration.customer_run_config_response.get('suppressions', [])
        if suppressions:
            secret_suppressions_ids = [
                suppression['policyId'] for suppression in suppressions
                if suppression['suppressionType'] == 'SecretsPolicy' or suppression['suppressionType'] == 'Policy'
            ]
            logging.info(f'The secret_suppressions_ids are: {secret_suppressions_ids}')

    return secret_suppressions_ids


def _find_files_from_root_folder(root_folder: str, runner_filter: RunnerFilter) -> List[str]:
    files_to_scan: List[str] = []
    excluded_paths = (runner_filter.excluded_paths or []) + EXCLUDED_PATHS
    enable_secret_scan_all_files = runner_filter.enable_secret_scan_all_files
    block_list_secret_scan = runner_filter.block_list_secret_scan or []
    block_list_secret_scan_lower = [file_type.lower() for file_type in block_list_secret_scan]
    for root, d_names, f_names in os.walk(root_folder):
        if enable_secret_scan_all_files:
            # 'excluded_paths' shouldn't include the static paths from 'EXCLUDED_PATHS'
            # they are separately referenced inside the 'filter_excluded_paths' function
            filter_excluded_paths(
                root_dir=root, names=d_names, excluded_paths=runner_filter.excluded_paths)
            filter_excluded_paths(
                root_dir=root, names=f_names, excluded_paths=runner_filter.excluded_paths)
        else:
            filter_ignored_paths(root, d_names, excluded_paths)
            filter_ignored_paths(root, f_names, excluded_paths)
        for file in f_names:
            if enable_secret_scan_all_files:
                if is_dockerfile(file):
                    if 'dockerfile' not in block_list_secret_scan_lower:
                        files_to_scan.append(os.path.join(root, file))
                elif f".{file.split('.')[-1]}" not in block_list_secret_scan_lower and file not in block_list_secret_scan_lower:
                    files_to_scan.append(os.path.join(root, file))
            elif file not in PROHIBITED_FILES and f".{file.split('.')[-1]}" in SUPPORTED_FILE_EXTENSIONS or is_dockerfile(
                    file):
                files_to_scan.append(os.path.join(root, file))
    logging.info(f'Secrets scanning will scan {len(files_to_scan)} files')

    return files_to_scan


def _cleanup_plugin_files(
        work_path: str,
        amount: int,
        dir_obj: Optional[tempfile.TemporaryDirectory[Any]] = None
) -> None:
    if dir_obj is not None:
        logging.info(f"Cleanup the whole temp directory: {work_path}")
        dir_obj.cleanup()
        return
    for index in range(1, amount):
        try:
            os.remove(f"{work_path}/runnable_plugin_{index}.py")
            logging.info(f"Removed runnable plugin at index {index}")
        except Exception as e:
            logging.info(f"Failed removing file at index {index} due to: {e}")


class Runner(BaseRunner[None, None, None]):
    check_type = CheckType.SECRETS  # noqa: CCE003  # a static attribute

    def __init__(
            self,
            file_extensions: Iterable[str] | None = None,
            file_names: Iterable[str] | None = None,
            entropy_limit: Optional[float] = None):
        super().__init__(file_extensions, file_names)
        self.commits_to_scan: Optional[List[Commit]] = None
        self.secrets_coordinator = SecretsCoordinator()
        self.history_secret_store = GitHistorySecretStore()
        self.entropy_limit = entropy_limit or float(os.getenv('CHECKOV_ENTROPY_KEYWORD_LIMIT', '3'))

    def set_history_secret_store(self, value: Dict[str, List[EnrichedPotentialSecret]]) -> None:
        self.history_secret_store.secrets_by_file_value_type = value

    def get_history_secret_store(self) -> Dict[str, List[EnrichedPotentialSecret]]:
        return self.history_secret_store.secrets_by_file_value_type

    def _get_plugins_used(self) -> Tuple[List[Dict[str, Any]], Callable[[], None]]:
        work_dir_obj = None
        work_path_optional = os.getenv('WORKDIR')
        if work_path_optional is None:
            work_dir_obj = tempfile.TemporaryDirectory()
            work_path = work_dir_obj.name
        else:
            work_path = work_path_optional

        # load runnable plugins
        current_dir = Path(__file__).parent
        plugins_used: List[Dict[str, Any]] = [
            {'name': 'AWSKeyDetector'},
            {'name': 'ArtifactoryDetector'},
            {'name': 'AzureStorageKeyDetector'},
            {'name': 'BasicAuthDetector'},
            {'name': 'CloudantDetector'},
            {'name': 'IbmCloudIamDetector'},
            {'name': 'IbmCosHmacDetector'},
            {'name': 'JwtTokenDetector'},
            {'name': 'MailchimpDetector'},
            {'name': 'NpmDetector'},
            {'name': 'PrivateKeyDetector'},
            {'name': 'SlackDetector'},
            {'name': 'SoftlayerDetector'},
            {'name': 'SquareOAuthDetector'},
            {'name': 'StripeDetector'},
            {'name': 'TwilioKeyDetector'},
            {'name': 'EntropyKeywordCombinator', 'path': f'file://{current_dir}/plugins/entropy_keyword_combinator.py',
             'entropy_limit': self.entropy_limit}
        ]
        plugins_index = 0
        if bc_integration.customer_run_config_response:
            policies_list = bc_integration.customer_run_config_response.get('secretsPolicies', [])
            if policies_list:
                runnable_plugins: dict[str, str] = get_runnable_plugins(policies_list)
                logging.debug(f"Found {len(runnable_plugins)} runnable plugins")
                if len(runnable_plugins) > 0:
                    plugins_index += 1
                for name, runnable_plugin in runnable_plugins.items():
                    f = open(f"{work_path}/runnable_plugin_{plugins_index}.py", "w")
                    f.write(runnable_plugin)
                    f.close()
                    plugins_used.append({
                        'name': name.replace(' ', ''),
                        'path': f'file://{work_path}/runnable_plugin_{plugins_index}.py'
                    })
                    plugins_index += 1
                    logging.debug(f"Loaded runnable plugin {name}")

        # load internal regex detectors
        detector_path = f"{current_dir}/plugins/custom_regex_detector.py"
        logging.info(f"Custom detector found at {detector_path}. Loading...")
        plugins_used.append({
            'name': 'CustomRegexDetector',
            'path': f'file://{detector_path}'
        })

        return plugins_used, lambda: _cleanup_plugin_files(work_path, plugins_index, work_dir_obj)

    def run(
            self,
            root_folder: str | None,
            external_checks_dir: list[str] | None = None,
            files: list[str] | None = None,
            runner_filter: RunnerFilter | None = None,
            collect_skip_comments: bool = True
    ) -> Report:
        runner_filter = runner_filter or RunnerFilter()
        secrets = SecretsCollection()

        plugins_used, cleanupFn = self._get_plugins_used()
        secret_suppressions_ids = _get_secret_suppressions_ids()

        if not runner_filter.show_progress_bar:
            self.pbar.turn_off_progress_bar()

        # Implement non IaC files (including .terraform dir)
        files_to_scan = files or []
        self._add_custom_detectors_to_metadata_integration()

        git_history_scanner = None
        if runner_filter.enable_git_history_secret_scan:
            git_history_scanner = GitHistoryScanner(str(root_folder), secrets, self.history_secret_store, runner_filter.git_history_timeout)

        with _thread_safe_transient_settings({
            # Only run scans with only these plugins.
            'plugins_used': plugins_used
        }) as settings:
            if root_folder:
                if runner_filter.enable_git_history_secret_scan and git_history_scanner is not None:
                    settings.disable_filters(*['detect_secrets.filters.common.is_invalid_file'])
                    git_history_scanner.scan_history(last_commit_scanned=runner_filter.git_history_last_commit_scanned, commits_to_scan=self.commits_to_scan)
                    logging.info(f'Secrets scanning git history for root folder {root_folder}')
                else:
                    files_to_scan += _find_files_from_root_folder(root_folder, runner_filter)

            settings.disable_filters(*['detect_secrets.filters.heuristic.is_indirect_reference'])
            settings.disable_filters(*['detect_secrets.filters.heuristic.is_potential_uuid'])

            if not runner_filter.enable_git_history_secret_scan:
                self.pbar.initiate(len(files_to_scan))
                self._scan_files(files_to_scan, secrets, self.pbar)
                self.pbar.close()

        history_store = None
        if runner_filter.enable_git_history_secret_scan and git_history_scanner is not None:
            history_store = git_history_scanner.history_store

        return self.get_report(secrets=secrets, runner_filter=runner_filter, history_store=history_store,
                               root_folder=root_folder, secret_suppressions_ids=secret_suppressions_ids, cleanupFn=cleanupFn)

    def get_report(self, secrets: SecretsCollection, runner_filter: RunnerFilter,
                   history_store: Optional[GitHistorySecretStore], root_folder: Optional[str],
                   secret_suppressions_ids: List[str], cleanupFn: Any, use_secret_filename: Optional[bool] = False) -> Report:
        report = Report(self.check_type)

        secret_records: dict[str, SecretsRecord] = {}
        secrets_in_uuid_form = ['CKV_SECRET_116', 'CKV_SECRET_49', 'CKV_SECRET_48', 'CKV_SECRET_40', 'CKV_SECRET_30']

        secret_key_by_line_to_secrets = defaultdict(list)
        for key, secret in secrets:
            secret_key_by_line_to_secrets[(key, secret.line_number)].append(secret)

        # If same line contains both Random High Entropy & Base64 High Entropy, only the Random one remains.
        # https://jira-dc.paloaltonetworks.com/browse/BCE-42547
        for secret_file_and_line_key, secrets_by_line in secret_key_by_line_to_secrets.items():
            if not any([s.check_id == RANDOM_HIGH_ENTROPY_CHECK_ID for s in secrets_by_line]):
                continue
            # Save resource id as we will need it for later
            entropy_secret = None
            _file_key = secret_file_and_line_key[0]
            for s in secrets_by_line:
                if SECRET_TYPE_TO_ID.get(s.type) == BASE64_HIGH_ENTROPY_CHECK_ID and entropy_secret is not None:
                    s.secret_value = entropy_secret
                if s.check_id == RANDOM_HIGH_ENTROPY_CHECK_ID and BASE64_HIGH_ENTROPY_CHECK_ID in [SECRET_TYPE_TO_ID.get(i.type) for i in secrets_by_line]:
                    try:
                        entropy_secret = s.secret_value if s.secret_value else None
                        secrets[_file_key].remove(s)
                    except KeyError:
                        pass

        for key, secret in secrets:
            check_id = secret.check_id if secret.check_id else SECRET_TYPE_TO_ID.get(secret.type)
            if not check_id:
                logging.debug(f'Secret was filtered - no check_id for line_number {secret.line_number}')
                continue
            if secret.secret_value and should_filter_vault_secret(secret.secret_value, check_id):
                logging.debug(f'Secret was filtered - this is a vault reference: {secret.secret_value}')
                continue
            secret_key = f'{key}_{secret.line_number}_{secret.secret_hash}'
            # secret history
            added_commit_hash, removed_commit_hash, code_line, added_by, removed_date, added_date = '', '', '', '', '', ''
            if runner_filter.enable_git_history_secret_scan and history_store is not None:
                enriched_potential_secret = history_store.get_added_and_removed_commit_hash(key, secret, root_folder)
                added_commit_hash = enriched_potential_secret.get('added_commit_hash') or ''
                removed_commit_hash = enriched_potential_secret.get('removed_commit_hash') or ''
                code_line = enriched_potential_secret.get('code_line') or ''
                added_by = enriched_potential_secret.get('added_by') or ''
                removed_date = enriched_potential_secret.get('removed_date') or ''
                added_date = enriched_potential_secret.get('added_date') or ''
            # run over secret key
            if isinstance(secret.secret_value, str) and secret.secret_value:
                stripped = secret.secret_value.strip(',";\'')
                if stripped != secret.secret_value:
                    secret_key = f'{key}_{secret.line_number}_{PotentialSecret.hash_secret(stripped)}'
            if secret.secret_value and is_potential_uuid(
                    secret.secret_value) and secret.check_id not in secrets_in_uuid_form:
                logging.info(
                    f"Removing secret due to UUID filtering: {PotentialSecret.hash_secret(secret.secret_value)}")
                continue
            bc_check_id = metadata_integration.get_bc_id(check_id)
            if bc_check_id in secret_suppressions_ids:
                logging.debug(f'Secret was filtered - check {check_id} was suppressed')
                continue
            severity = metadata_integration.get_severity(check_id)
            if not runner_filter.should_run_check(check_id=check_id, bc_check_id=bc_check_id, severity=severity,
                                                  report_type=CheckType.SECRETS):
                logging.debug(
                    f'Check was suppress - should_run_check. check_id {check_id}')
                continue
            if secret_key in secret_records.keys():
                is_prioritise = self._prioritise_secrets(secret_records, secret_key, check_id)
                if not is_prioritise:
                    continue
            result: _CheckResult = {'result': CheckResult.FAILED}
            try:
                if runner_filter.enable_git_history_secret_scan and code_line is not None:
                    line_text = code_line
                else:
                    line_text = linecache.getline(secret.filename, secret.line_number)
            except SyntaxError as e:
                # If encoding is a problem, this is probably not human-readable source code
                # hence there's no need in flagging this secret
                logging.info(f'Failed to log secret {secret.type} for file {secret.filename} because of {e}')
                continue
            if line_text and line_text.startswith('git_commit'):
                continue
            result = self.search_for_suppression(
                check_id=check_id,
                bc_check_id=bc_check_id,
                severity=severity,
                secret=secret,
                runner_filter=runner_filter,
                root_folder=root_folder
            ) or result

            relative_file_path = f'/{os.path.relpath(secret.filename, root_folder)}'
            if use_secret_filename:
                relative_file_path = f'/{secret.filename}'

            resource = f'{relative_file_path}:{added_commit_hash}:{secret.secret_hash}' if added_commit_hash else f'{relative_file_path}:{secret.secret_hash}'
            report.add_resource(resource)
            # 'secret.secret_value' can actually be 'None', but only when 'PotentialSecret' was created
            # via 'load_secret_from_dict'
            self.save_secret_to_coordinator(
                secret.secret_value, bc_check_id, check_id, resource, secret.line_number, result
            )

            secret_key_by_line = (key, secret.line_number)
            line_text_censored = line_text
            for sec in secret_key_by_line_to_secrets[secret_key_by_line]:
                secret_value = cast(str, sec.secret_value)
                if secret_value:
                    secret_value = secret_value.strip('"\'')  # We should always strip quotes from matches before we search for them in the line (because of this line quoted_mm = f"'{mm}'" in custom_regex_detector.py)
                line_text_censored = omit_secret_value_from_line(secret_value, line_text_censored)

            secret_records[secret_key] = SecretsRecord(
                check_id=check_id,
                bc_check_id=bc_check_id,
                severity=severity,
                check_name=secret.type,
                check_result=result,
                code_block=[(secret.line_number, line_text_censored)],
                file_path=relative_file_path,
                file_line_range=[secret.line_number, secret.line_number + 1],
                resource=f'{added_commit_hash}:{secret.secret_hash}' if added_commit_hash else secret.secret_hash,
                check_class="",
                evaluations=None,
                file_abs_path=os.path.abspath(secret.filename),
                validation_status=ValidationStatus.UNAVAILABLE.value,
                added_commit_hash=added_commit_hash,
                removed_commit_hash=removed_commit_hash,
                added_by=added_by,
                removed_date=removed_date,
                added_date=added_date
            )
        for _, v in secret_records.items():
            report.add_record(v)

        enriched_secrets_s3_path = bc_integration.persist_enriched_secrets(self.secrets_coordinator.get_secrets())
        if enriched_secrets_s3_path:
            self.verify_secrets(report, enriched_secrets_s3_path)
        logging.debug(f'report fail checks len: {len(report.failed_checks)}')

        if cleanupFn is not None:
            cleanupFn()
        if runner_filter.skip_invalid_secrets:
            self._modify_invalid_secrets_check_result_to_skipped(report)
        return report

    @staticmethod
    def _prioritise_secrets(secret_records: Dict[str, SecretsRecord], secret_key: str, check_id: str) -> bool:
        if secret_records[secret_key].check_id in ENTROPY_CHECK_IDS and check_id not in ENTROPY_CHECK_IDS:
            secret_records.pop(secret_key)
            return True
        if secret_records[secret_key].check_id in GENERIC_PRIVATE_KEY_CHECK_IDS:
            if check_id not in GENERIC_PRIVATE_KEY_CHECK_IDS | ENTROPY_CHECK_IDS:
                secret_records.pop(secret_key)
                return True
        if secret_records[secret_key].check_id == GENERIC_AWS_CHECK_ID and check_id in SPECIFIC_AWS_CHECK_IDS:
            secret_records.pop(secret_key)
            return True
        return False

    @staticmethod
    def _scan_files(files_to_scan: list[str], secrets: SecretsCollection, pbar: ProgressBar) -> None:
        # implemented the scan function like secrets.scan_files
        base_path = secrets.root
        items = [
            (file, base_path)
            for file in files_to_scan
        ]
        results = parallel_runner.run_function(func=Runner._safe_scan, items=items)

        for filename, secrets_results in results:
            pbar.set_additional_data({'Current File Scanned': str(filename)})
            for secret in secrets_results:
                secrets[os.path.relpath(secret.filename, base_path)].add(secret)
            pbar.update()

    @staticmethod
    def _prepare_scan_file(full_file_path: str) -> Optional[str]:
        """Check if a file has build log prefixes and prepare a stripped version for scanning.

        Build log files often have timestamps/log-level prefixes on each line that break
        multiline secret detection (e.g., private keys split across prefixed log lines).
        This method detects such prefixes and creates a temporary file with the prefixes
        stripped, so all detectors can match secrets that span multiple log lines.

        Returns:
            path to the temp file if created (caller must clean up), or None
        """
        stripped_content = create_stripped_content(full_file_path)
        if not stripped_content:
            return None

        logging.debug(f'Detected log prefixes in {full_file_path}, scanning with prefixes stripped')
        try:
            _, ext = os.path.splitext(full_file_path)
            with tempfile.NamedTemporaryFile(
                    mode='w', suffix=ext, delete=False
            ) as tmp_file:
                tmp_file.write(stripped_content)
                return tmp_file.name
        except Exception as e:
            logging.debug(f'Failed to create stripped temp file for {full_file_path}: {e}')
            return None

    @staticmethod
    def _safe_scan(file_path: str, base_path: str) -> tuple[str, list[PotentialSecret]]:
        try:
            full_file_path = os.path.join(base_path, file_path)
            file_size = os.path.getsize(full_file_path)
            if file_size > MAX_FILE_SIZE > 0:
                logging.info(
                    f'Skipping secret scanning on {full_file_path} due to file size. To scan this file for '
                    'secrets, run this command again with the environment variable "CHECKOV_MAX_FILE_SIZE" '
                    f'to 0 or {file_size + 1}'
                )
                return file_path, []

            start_time = datetime.datetime.now()

            tmp_file_path = Runner._prepare_scan_file(full_file_path)
            scan_file_path = tmp_file_path or full_file_path

            try:
                file_results = [*scan.scan_file(scan_file_path)]
                # If we scanned a temp file, map results back to the original file path
                if tmp_file_path is not None:
                    for secret in file_results:
                        secret.filename = full_file_path
                logging.debug(f'file {full_file_path} results len {len(file_results)}')
            finally:
                if tmp_file_path is not None:
                    try:
                        os.remove(tmp_file_path)
                    except OSError:
                        logging.warning(f"Failed to remove temp file: {tmp_file_path}")

            end_time = datetime.datetime.now()
            run_time = end_time - start_time
            if run_time > datetime.timedelta(seconds=10):
                logging.info(f'Secret scanning for {full_file_path} took {run_time} seconds')
            return file_path, file_results
        except Exception as e:
            logging.warning(f"Secret scanning: could not process file {full_file_path}")
            logging.debug(e, exc_info=True)
            return file_path, []

    @staticmethod
    def search_for_suppression(
            check_id: str,
            bc_check_id: str,
            severity: Severity | None,
            secret: PotentialSecret,
            runner_filter: RunnerFilter,
            root_folder: str | None
    ) -> _CheckResult | None:
        if not runner_filter.should_run_check(
                check_id=check_id,
                bc_check_id=bc_check_id,
                severity=severity,
                report_type=CheckType.SECRETS,
                file_origin_paths=[secret.filename],
                root_folder=root_folder
        ) and check_id in CHECK_ID_TO_SECRET_TYPE.keys():
            return {
                "result": CheckResult.SKIPPED,
                "suppress_comment": f"Secret scan {check_id} is skipped"
            }

        # Check for suppression comment in the line before, the line of, and the line after the secret
        for line_number in [secret.line_number, secret.line_number - 1, secret.line_number + 1]:
            lt = linecache.getline(secret.filename, line_number)
            skip_search = re.search(COMMENT_REGEX, lt)
            if skip_search and (skip_search.group(2) == check_id or skip_search.group(2) == bc_check_id):
                comment: str = skip_search.group(3)[1:] if skip_search.group(3) else "No comment provided"
                return {
                    "result": CheckResult.SKIPPED,
                    "suppress_comment": comment
                }

            # Metadata suppression check
            try:
                secret_file_path = os.path.join(root_folder, secret.filename) if root_folder else secret.filename
                parser = ContextParser(secret_file_path)

                # Parse the file for metadata
                resource_config = getattr(secret, "resource_config", None)
                if resource_config is None and secret_file_path.endswith((".json", ".yml", ".yaml")):
                    try:
                        import json
                        import yaml
                        with open(secret_file_path, "r", encoding="utf-8") as f:
                            content = f.read()
                            if secret_file_path.endswith(".json"):
                                resource_config = json.loads(content)
                            else:
                                resource_config = yaml.safe_load(content)

                        if not isinstance(resource_config, (dict, list)):
                            resource_config = None
                    except Exception:
                        resource_config = None

                suppressions = parser.collect_skip_comments(resource_config=resource_config)
                metadata_suppressions = [s for s in suppressions if s.get("line_number") is None]

                for suppression in metadata_suppressions:
                    if suppression["id"] == check_id or suppression.get("bc_id") == bc_check_id:
                        return {
                            "result": CheckResult.SKIPPED,
                            "suppress_comment": suppression.get("suppress_comment", "No comment provided")
                        }

            except Exception as e:
                logging.debug(f"Metadata suppression check failed for file {secret.filename}: {e}")
        return None

    def save_secret_to_coordinator(
            self,
            secret_value: Optional[str],
            bc_check_id: str,
            check_id: str,
            resource: str,
            line_number: int,
            result: _CheckResult
    ) -> None:
        if result.get('result') == CheckResult.FAILED and secret_value is not None:
            enriched_secret = EnrichedSecret(
                original_secret=secret_value,
                bc_check_id=bc_check_id,
                check_id=check_id,
                resource=resource,
                line_number=line_number
            )
            self.secrets_coordinator.add_secret(enriched_secret=enriched_secret)

    @time_it
    def verify_secrets(self, report: Report, enriched_secrets_s3_path: str) -> VerifySecretsResult:
        if not bc_integration.bc_api_key:
            logging.debug('Secrets verification is available only with a valid API key')
            return VerifySecretsResult.INSUFFICIENT_PARAMS

        if bc_integration.skip_download:
            logging.debug('Skipping secrets verification as flag skip-download was specified')
            return VerifySecretsResult.INSUFFICIENT_PARAMS

        validate_secrets_tenant_config = None
        if bc_integration.customer_run_config_response is not None:
            validate_secrets_tenant_config = bc_integration.customer_run_config_response.get(
                'tenantConfig', {}).get('secretsValidate')

        if validate_secrets_tenant_config is None and not convert_str_to_bool(os.getenv("CKV_VALIDATE_SECRETS", False)):
            logging.debug('Secrets verification is off. enable it via code configuration screen')
            return VerifySecretsResult.INSUFFICIENT_PARAMS

        if validate_secrets_tenant_config is False:
            logging.debug('Secrets verification is off. enable it via code configuration screen')
            return VerifySecretsResult.INSUFFICIENT_PARAMS

        request_body = {
            "reportS3Path": enriched_secrets_s3_path
        }
        response = None
        try:
            response = request_wrapper(
                "POST", f"{bc_integration.api_url}/api/v1/secrets/reportVerification",
                headers=bc_integration.get_default_headers("POST"),
                json=request_body,
                should_call_raise_for_status=True,
                log_json_body=False
            )
        except Exception:
            logging.error('Failed to perform secrets verification', exc_info=True)

        if not response:
            return VerifySecretsResult.FAILURE

        verification_report_presigned_url = response.json().get('verificationReportSignedUrl')
        if not verification_report_presigned_url:
            logging.error("Response is missing verificationReportSignedUrl key, aborting")
            return VerifySecretsResult.FAILURE

        verification_report = self.get_json_verification_report(verification_report_presigned_url)

        if not verification_report:
            return VerifySecretsResult.FAILURE

        validation_status_by_check_id_and_resource = {}
        for validation_status_entity in verification_report:
            if not all(required_key in validation_status_entity.keys() for required_key in
                       ["violationId", "resourceId", "status"]):
                logging.debug(f"{validation_status_entity} does not have all required keys, skipping")
                continue

            key = f'{validation_status_entity["violationId"]}_{validation_status_entity["resourceId"]}'
            validation_status_by_check_id_and_resource[key] = validation_status_entity['status']

        logging.debug(
            f'secrets verification api returned with {len(validation_status_by_check_id_and_resource.keys())} unique entries')

        for secrets_record in report.failed_checks:
            if hasattr(secrets_record, "validation_status"):
                key = f'{secrets_record.bc_check_id}_{secrets_record.file_path}:{secrets_record.resource}'
                secrets_record.validation_status = validation_status_by_check_id_and_resource.get(key)

                if secrets_record.validation_status is None:
                    logging.debug(f'Failed to find verification status of {key}, setting by default to Unknown')
                    secrets_record.validation_status = ValidationStatus.UNAVAILABLE.value

        return VerifySecretsResult.SUCCESS

    @staticmethod
    def get_json_verification_report(presigned_url: str) -> list[dict[str, str]] | None:
        response = None
        try:
            response = requests.get(url=presigned_url, timeout=DEFAULT_TIMEOUT)
        except Exception:
            logging.error('Unable to download verification report')

        return response.json() if response else None

    @staticmethod
    def _add_custom_detectors_to_metadata_integration() -> None:
        customer_run_config_response = bc_integration.customer_run_config_response
        policies_list: List[dict[str, Any]] = []
        if customer_run_config_response:
            policies_list = customer_run_config_response.get('secretsPolicies', [])
        for policy in policies_list:
            if policy.get('isCustom', False):
                check_id = policy['incidentId']
                guideline = policy.get('guideline', '')
                severity = policy.get('severity', '')
                metadata_integration.check_metadata[check_id] = {'id': check_id,
                                                                 'guideline': guideline,
                                                                 'severity': severity}

    @staticmethod
    def _modify_invalid_secrets_check_result_to_skipped(report: Report) -> None:
        checks_indexes_moved_to_skipped: list[int] = []

        for check_index, check in enumerate(report.failed_checks):
            if hasattr(check, 'validation_status') and check.validation_status == ValidationStatus.INVALID.value:
                check.check_result["result"] = CheckResult.SKIPPED
                check.check_result["suppress_comment"] = "Skipped invalid secret"
                report.skipped_checks.append(check)
                checks_indexes_moved_to_skipped.append(check_index)

        for idx in sorted(checks_indexes_moved_to_skipped, reverse=True):
            try:
                del report.failed_checks[idx]
            except Exception:
                logging.error(f"Failed to remove suppressed secrets violations from failed_checks, report is corrupted."
                              f"Tried to delete entry {idx} from failed_checks of length {len(report.failed_checks)}",
                              exc_info=True)

    def mask_files(self, root_folder: str | None,
                   files: list[str] | None = None,
                   runner_filter: RunnerFilter | None = None) -> None:
        """
        get files or/and root_folder and masking and replace automatically all the secrets found there
        note: the changes are inplace
        """
        runner_filter = runner_filter or RunnerFilter()

        plugins_used, cleanupFn = self._get_plugins_used()

        files_to_scan = files or []
        if root_folder:
            files_to_scan += _find_files_from_root_folder(root_folder, runner_filter)

        self._add_custom_detectors_to_metadata_integration()
        secrets = SecretsCollection()
        with _thread_safe_transient_settings({
            # Only run scans with only these plugins.
            'plugins_used': plugins_used
        }) as settings:

            settings.disable_filters(*['detect_secrets.filters.heuristic.is_indirect_reference'])
            settings.disable_filters(*['detect_secrets.filters.heuristic.is_potential_uuid'])

            self._scan_files(files_to_scan, secrets, self.pbar)

        for file in files_to_scan:
            with open(file, "r+") as f:
                content = f.read()
                f.seek(0)
                for _key, secret in secrets:
                    if not secret.secret_value:
                        continue
                    check_id = secret.check_id if secret.check_id else SECRET_TYPE_TO_ID.get(secret.type)
                    if not check_id:
                        logging.debug(f'Secret was filtered - no check_id for line_number {secret.line_number}')
                        continue
                    if secret.secret_value and should_filter_vault_secret(secret.secret_value, check_id):
                        logging.debug(f'Secret was filtered - this is a vault reference: {secret.secret_value}')
                        continue

                    content = content.replace(secret.secret_value, masking_value(secret.secret_value))
                f.write(content)
                f.truncate()

        logging.info(f"finish replacing {len(files_to_scan)} files")


def masking_value(secret: str) -> str:
    secret_len_to_expose = min(len(secret) // 4, 6)
    return f'{secret[:secret_len_to_expose]}{"*" * GENERIC_OBFUSCATION_LENGTH}'



from typing import Dict, List, Any

from checkov.common.models.enums import CheckResult, CheckCategories
from checkov.terraform.checks.resource.base_resource_check import BaseResourceCheck


class VMDiskWithPublicAccess(BaseResourceCheck):
    def __init__(self) -> None:
        name = "Ensure Azure Virtual Machine disks are configured without public network access"
        id = "CKV_AZURE_251"
        supported_resources = ['azurerm_managed_disk']
        categories = [CheckCategories.NETWORKING]
        super().__init__(name=name, id=id, categories=categories, supported_resources=supported_resources)

    def scan_resource_conf(self, conf: Dict[str, Any]) -> CheckResult:
        if "public_network_access_enabled" in conf:
            if "True" in conf["public_network_access_enabled"] or True in conf["public_network_access_enabled"]:
                return CheckResult.FAILED
            else:
                return CheckResult.PASSED
        else:
            return CheckResult.FAILED

    def get_evaluated_keys(self) -> List[str]:
        return ['public_network_access_enabled']


check = VMDiskWithPublicAccess()



import platform
import sys
import unittest
import re
from pathlib import Path

from checkov.common.models.consts import ckv_check_id_pattern

current_dir = Path(__file__).parent


class TestCheckovPlatformOnlyPolicies(unittest.TestCase):

    def test_no_ckv_ids_api_key(self):
        checks_list_path = current_dir.parent / 'checkov_checks_list.txt'
        if sys.version_info[1] == 8 and platform.system() == 'Linux':
            with open(checks_list_path, encoding='utf-8') as f:
                for i, line in enumerate(f):
                    if i in [0, 1]:
                        # skip the header lines
                        continue
                    line = "".join(line.split())
                    if line and isinstance(line, str):
                        if line == "---":
                            # end of table
                            continue
                        check_id = line.split('|')[2]
                        ckv_ids = re.match(ckv_check_id_pattern, check_id)
                        self.assertFalse(ckv_ids)


if __name__ == '__main__':
    unittest.main()



import os
import unittest

from checkov.arm.runner import Runner
from checkov.runner_filter import RunnerFilter


class TestSkipJsonRegexPattern(unittest.TestCase):

    def test_skip_all_checks(self):
        runner = Runner()
        current_dir = os.path.dirname(os.path.realpath(__file__))

        test_files_dir = os.path.join(current_dir, "example_SkipJsonRegex")
        report = runner.run(
            root_folder=test_files_dir,
            runner_filter=RunnerFilter(skip_checks=["CKV_AZURE_*:.*.json$"])
        )

        summary = report.get_summary()

        self.assertEqual(summary['passed'], 0)
        self.assertEqual(summary['failed'], 0)
        self.assertEqual(summary['skipped'], 0)
        self.assertEqual(summary['parsing_errors'], 0)

    def test_skip_specific_check(self):
        runner = Runner()
        current_dir = os.path.dirname(os.path.realpath(__file__))

        test_files_dir = os.path.join(current_dir, "example_SkipJsonRegex")
        report = runner.run(
            root_folder=test_files_dir,
            runner_filter=RunnerFilter(skip_checks=["CKV_AZURE_8:.*.json$"])
        )

        summary = report.get_summary()

        self.assertEqual(summary['passed'], 4)
        self.assertEqual(summary['failed'], 40)  # Updated expected value
        self.assertEqual(summary['skipped'], 0)
        self.assertEqual(summary['parsing_errors'], 0)

    def test_skip_specific_check_for_folder(self):
        runner = Runner()
        current_dir = os.path.dirname(os.path.realpath(__file__))

        test_files_dir = os.path.join(current_dir, "example_SkipJsonRegex")
        report = runner.run(
            root_folder=test_files_dir,
            runner_filter=RunnerFilter(skip_checks=["CKV_AZURE_8:/skip2.[a-z1-9]*.json$"])
        )

        summary = report.get_summary()

        self.assertEqual(summary['passed'], 4)
        self.assertEqual(summary['failed'], 42)  # Updated expected value
        self.assertEqual(summary['skipped'], 0)
        self.assertEqual(summary['parsing_errors'], 0)

    def test_skip_specific_check_specific_file(self):
        runner = Runner()
        current_dir = os.path.dirname(os.path.realpath(__file__))

        test_files_dir = os.path.join(current_dir, "example_SkipJsonRegex")
        report = runner.run(
            root_folder=test_files_dir,
            runner_filter=RunnerFilter(skip_checks=["CKV_AZURE_8:/.*skip1.json$"])
        )

        summary = report.get_summary()

        self.assertEqual(summary['passed'], 4)
        self.assertEqual(summary['failed'], 42)  # Updated expected value
        self.assertEqual(summary['skipped'], 0)
        self.assertEqual(summary['parsing_errors'], 0)

    def test_no_skip(self):
        runner = Runner()
        current_dir = os.path.dirname(os.path.realpath(__file__))

        test_files_dir = os.path.join(current_dir, "example_SkipJsonRegex")
        report = runner.run(
            root_folder=test_files_dir,
            runner_filter=RunnerFilter(skip_checks=["CKV_AZURE_*:/.*skip555.json$"])
        )

        summary = report.get_summary()

        self.assertEqual(summary['passed'], 4)
        self.assertEqual(summary['failed'], 44)  # Updated expected value
        self.assertEqual(summary['skipped'], 0)
        self.assertEqual(summary['parsing_errors'], 0)


if __name__ == '__main__':
    unittest.main()
